@@ -5,6 +5,7 @@
 #include <CGAL/Three/Scene_print_item_interface.h>
 #include <CGAL/Three/Scene_transparent_interface.h>
 #include <CGAL/Three/Scene_zoomable_item_interface.h>
+#include <CGAL/Three/Three.h>
 
 #include  <CGAL/Three/Scene_item.h>
 #include <CGAL/Three/Scene_print_item_interface.h>
@@ -26,6 +27,7 @@
 #include <QMimeData>
 #include <QOpenGLFramebufferObject>
 
+#include <unordered_set>
 
 Scene::Scene(QObject* parent)
     : QStandardItemModel(parent),
@@ -81,6 +83,16 @@ Scene::addItem(CGAL::Three::Scene_item* item)
             qobject_cast<CGAL::Three::Scene_group_item*>(item);
     if(group)
         addGroup(group);
+    //init the item for the mainViewer to avoid using unexisting
+    //VAOs if the mainViewer is not the first to be drawn.
+    QOpenGLFramebufferObject* fbo = CGAL::Three::Three::mainViewer()->depthPeelingFbo();
+    CGAL::Three::Three::mainViewer()->setDepthPeelingFbo(nullptr);//to prevent crashing as the fbo is not initialized in this call.
+    item->draw(CGAL::Three::Three::mainViewer());
+    item->drawEdges(CGAL::Three::Three::mainViewer());
+    item->drawPoints(CGAL::Three::Three::mainViewer());
+    CGAL::Three::Three::mainViewer()->setDepthPeelingFbo(fbo);
+    if(group)
+       m_groups.append(id);
     return id;
 }
 
@@ -88,7 +100,7 @@ CGAL::Three::Scene_item*
 Scene::replaceItem(Scene::Item_id index, CGAL::Three::Scene_item* item, bool emit_item_about_to_be_destroyed)
 {
     if(index < 0 || index >= m_entries.size())
-        return 0;
+        return nullptr;
 
     connect(item, SIGNAL(itemChanged()),
             this, SLOT(itemChanged()));
@@ -101,6 +113,7 @@ Scene::replaceItem(Scene::Item_id index, CGAL::Three::Scene_item* item, bool emi
     QList<Scene_item*> group_children;
     if(group)
     {
+      m_groups.removeAll(index);
       Q_FOREACH(Item_id id, group->getChildren())
       {
         CGAL::Three::Scene_item* child = group->getChild(id);
@@ -134,6 +147,7 @@ Scene::replaceItem(Scene::Item_id index, CGAL::Three::Scene_item* item, bool emi
 
     if(emit_item_about_to_be_destroyed) {
       Q_EMIT itemAboutToBeDestroyed(item);
+      item->aboutToBeDestroyed();
     }
 
     Q_EMIT updated();
@@ -142,6 +156,7 @@ Scene::replaceItem(Scene::Item_id index, CGAL::Three::Scene_item* item, bool emi
     if(group)
     {
         addGroup(group);
+        m_groups.append(index);
     }
     itemChanged(index);
     Q_EMIT restoreCollapsedState();
@@ -157,20 +172,22 @@ Scene::replaceItem(Scene::Item_id index, CGAL::Three::Scene_item* item, bool emi
 Scene::Item_id
 Scene::erase(Scene::Item_id index)
 {
+  if(index <0 || index >= numberOfEntries())
+    return -1;
+
   CGAL::Three::Scene_item* item = m_entries[index];
   if(qobject_cast<Scene_group_item*>(item))
   {
     setSelectedItemsList(QList<Scene_interface::Item_id>()<<item_id(item));
     return erase(selectionIndices());
   }
+  m_groups.removeAll(index);
   if(item->parentGroup()
      && item->parentGroup()->isChildLocked(item))
     return -1;
   //clears the Scene_view
   clear();
   index_map.clear();
-  if(index < 0 || index >= m_entries.size())
-    return -1;
   if(item->parentGroup())
     item->parentGroup()->removeChild(item);
 
@@ -178,7 +195,7 @@ Scene::erase(Scene::Item_id index)
   Item_id removed_item = item_id(item);
   children.removeAll(removed_item);
   indexErased(removed_item);
-    m_entries.removeAll(item);
+  m_entries.removeAll(item);
   Q_EMIT itemAboutToBeDestroyed(item);
   item->aboutToBeDestroyed();
   item->deleteLater();
@@ -204,7 +221,7 @@ Scene::erase(QList<int> indices)
 {
   if(indices.empty())
     return -1;
-  QList<CGAL::Three::Scene_item*> to_be_removed;
+  std::unordered_set<CGAL::Three::Scene_item*> to_be_removed;
   int max_index = -1;
   Q_FOREACH(int index, indices) {
     if(index < 0 || index >= m_entries.size())
@@ -222,15 +239,13 @@ Scene::erase(QList<int> indices)
       Q_FOREACH(Item_id id, group->getChildren())
       {
         CGAL::Three::Scene_item* child = group->getChild(id);
-        if(!to_be_removed.contains(child))
-          to_be_removed.push_back(child);
+        to_be_removed.insert(child);
       }
     }
-    if(!to_be_removed.contains(item))
-      to_be_removed.push_back(item);
+    to_be_removed.insert(item);
   }
 
-  Q_FOREACH(Scene_item* item, to_be_removed) {
+  for(Scene_item* item : to_be_removed) {
     Item_id removed_item = item_id(item);
     if(removed_item == -1) //case of the selection_item, for example.
       continue;
@@ -238,8 +253,9 @@ Scene::erase(QList<int> indices)
       item->parentGroup()->removeChild(item);
     children.removeAll(removed_item);
     indexErased(removed_item);
+    m_groups.removeAll(removed_item);
     m_entries.removeAll(item);
-    
+
     Q_EMIT itemAboutToBeDestroyed(item);
     item->aboutToBeDestroyed();
     item->deleteLater();
@@ -279,8 +295,16 @@ void Scene::remove_item_from_groups(Scene_item* item)
 }
 Scene::~Scene()
 {
-  vao->destroy();
-  delete vao;  
+  Q_FOREACH(CGAL::QGLViewer* viewer, CGAL::QGLViewer::QGLViewerPool())
+  {
+    removeViewer(static_cast<CGAL::Three::Viewer_interface*>(viewer));
+    viewer->setProperty("is_destroyed", true);
+  }
+  Q_FOREACH(QOpenGLVertexArrayObject* vao, vaos.values())
+  {
+    vao->destroy();
+    delete vao;
+  }
   Q_FOREACH(CGAL::Three::Scene_item* item_ptr, m_entries)
   {
     item_ptr->deleteLater();
@@ -294,7 +318,7 @@ Scene::item(Item_id index) const
     return m_entries.value(index); // QList::value checks bounds
 }
 
-Scene::Item_id 
+Scene::Item_id
 Scene::item_id(CGAL::Three::Scene_item* scene_item) const
 {
     return m_entries.indexOf(scene_item);
@@ -329,7 +353,7 @@ Scene::duplicate(Item_id index)
 
 void Scene::initializeGL(CGAL::Three::Viewer_interface* viewer)
 {
-  
+
   //Vertex source code
   const char vertex_source[] =
   {
@@ -343,9 +367,9 @@ void Scene::initializeGL(CGAL::Three::Viewer_interface* viewer)
     "  f_texCoord = v_texCoord;                  \n"
     "  gl_Position = projection_matrix * vertex; \n"
     "}                                           \n"
-    
+
   };
-  
+
   const char vertex_source_comp[] =
   {
     "attribute highp vec4 vertex;                \n"
@@ -357,7 +381,7 @@ void Scene::initializeGL(CGAL::Three::Viewer_interface* viewer)
     "  f_texCoord = v_texCoord;                  \n"
     "  gl_Position = projection_matrix * vertex; \n"
     "}                                           \n"
-    
+
   };
   //Fragment source code
   const char fragment_source[] =
@@ -380,8 +404,8 @@ void Scene::initializeGL(CGAL::Three::Viewer_interface* viewer)
     "  gl_FragColor = texture2D(texture, f_texCoord); \n"
     "}                                                                      \n"
   };
-  
-  
+
+
   QOpenGLShader vertex_shader(QOpenGLShader::Vertex);
   QOpenGLShader fragment_shader(QOpenGLShader::Fragment);
   if(viewer->isOpenGL_4_3())
@@ -390,7 +414,7 @@ void Scene::initializeGL(CGAL::Three::Viewer_interface* viewer)
     {
       std::cerr<<"Compiling vertex source FAILED"<<std::endl;
     }
-    
+
     if(!fragment_shader.compileSourceCode(fragment_source))
     {
       std::cerr<<"Compiling fragmentsource FAILED"<<std::endl;
@@ -402,13 +426,13 @@ void Scene::initializeGL(CGAL::Three::Viewer_interface* viewer)
     {
       std::cerr<<"Compiling vertex source FAILED"<<std::endl;
     }
-    
+
     if(!fragment_shader.compileSourceCode(fragment_source_comp))
     {
       std::cerr<<"Compiling fragmentsource FAILED"<<std::endl;
     }
   }
-  
+
   if(!program.addShader(&vertex_shader))
   {
     std::cerr<<"adding vertex shader FAILED"<<std::endl;
@@ -428,34 +452,34 @@ void Scene::initializeGL(CGAL::Three::Viewer_interface* viewer)
   points[9] = -1.0f; points[10] = -1.0f; points[11] = 0.0f;
   points[12] = -1.0f; points[13] = 1.0f; points[14] = 0.0f;
   points[15] = 1.0f; points[16] = 1.0f; points[17] = 0.0f;
-  
+
   uvs[0] = 0.0f; uvs[1] = 0.0f;
   uvs[2] = 1.0f; uvs[3] = 1.0f;
   uvs[4] = 1.0f; uvs[5] = 0.0f;
   uvs[6] = 0.0f; uvs[7] = 0.0f;
   uvs[8] = 0.0f; uvs[9] = 1.0f;
   uvs[10] = 1.0f; uvs[11] = 1.0f;
-  
+
   vbo[0].create();
   vbo[1].create();
-  
+
   viewer->makeCurrent();
-  vao = new QOpenGLVertexArrayObject();
-  vao->create();
+  vaos[viewer] = new QOpenGLVertexArrayObject();
+  vaos[viewer]->create();
   program.bind();
-  vao->bind();
+  vaos[viewer]->bind();
   vbo[0].bind();
   vbo[0].allocate(points, 18 * sizeof(float));
   program.enableAttributeArray("vertex");
-  program.setAttributeArray("vertex", GL_FLOAT, 0, 3);
+  program.setAttributeArray("vertex", GL_FLOAT, nullptr, 3);
   vbo[0].release();
-  
+
   vbo[1].bind();
   vbo[1].allocate(uvs, 12 * sizeof(float));
   program.enableAttributeArray("v_texCoord");
-  program.setAttributeArray("v_texCoord", GL_FLOAT, 0, 2);
+  program.setAttributeArray("v_texCoord", GL_FLOAT, nullptr, 2);
   vbo[1].release();
-  vao->release();
+  vaos[viewer]->release();
   program.release();
   gl_init = true;
 }
@@ -495,13 +519,13 @@ bool item_should_be_skipped_in_draw(Scene_item* item) {
   if(!item->visible()) return true;
   if(item->has_group == 0) return false;
   Scene_group_item* group = item->parentGroup();
-  while(group != 0) {
+  while(group != nullptr) {
     if(!group->visible()) return false;
     group = group->parentGroup();
   }
   return true;
 }
- 
+
 
 void Scene::renderScene(const QList<Scene_interface::Item_id> &items,
                         Viewer_interface *viewer,
@@ -511,7 +535,6 @@ void Scene::renderScene(const QList<Scene_interface::Item_id> &items,
                         bool writing_depth,
                         QOpenGLFramebufferObject *fbo)
 {
-
   viewer->setCurrentPass(pass);
   viewer->setDepthWriting(writing_depth);
   viewer->setDepthPeelingFbo(fbo);
@@ -525,27 +548,23 @@ void Scene::renderScene(const QList<Scene_interface::Item_id> &items,
       item.selection_changed(true);
     }
     else
-
     {
       item.selection_changed(false);
     }
-
     if(group ||item.visible())
     {
-      if( group || item.renderingMode() == Flat || item.renderingMode() == FlatPlusEdges || item.renderingMode() == Gouraud)
+      if( group || item.renderingMode() == Flat || item.renderingMode() == FlatPlusEdges || item.renderingMode() == Gouraud || item.renderingMode() == GouraudPlusEdges )
       {
         if(with_names) {
           viewer->glClearDepthf(1.0);
           viewer->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
         }
         item.draw(viewer);
-
         if(with_names) {
-          
+
           //    read depth buffer at pick location;
-          float depth = 1.0;
-          viewer->glReadPixels(picked_pixel.x(),viewer->camera()->screenHeight()-1-picked_pixel.y(),1,1,GL_DEPTH_COMPONENT, GL_FLOAT, &depth);
-          if (depth != 1.0)
+          float depth = read_depth_under_pixel(picked_pixel, viewer, viewer->camera());
+          if (depth < 2.0)
           {
             //add object to list of picked objects;
             picked_item_IDs[depth] = index;
@@ -581,8 +600,13 @@ void Scene::renderWireScene(const QList<Scene_interface::Item_id> &items,
      {
        if( group || (!with_names && item.renderingMode() == FlatPlusEdges )
           || item.renderingMode() == Wireframe
-          || item.renderingMode() == PointsPlusNormals)
+          || item.renderingMode() == PointsPlusNormals
+          || item.renderingMode() == GouraudPlusEdges)
        {
+         if(with_names) {
+           viewer->glClearDepthf(1.0);
+           viewer->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+         }
          viewer->setGlPointSize(2.f);
          item.drawEdges(viewer);
        }
@@ -609,7 +633,7 @@ void Scene::renderWireScene(const QList<Scene_interface::Item_id> &items,
 
          //    read depth buffer at pick location;
          float depth = 1.0;
-         viewer->glReadPixels(picked_pixel.x(),viewer->camera()->screenHeight()-1-picked_pixel.y(),1,1,GL_DEPTH_COMPONENT, GL_FLOAT, &depth);
+         depth = read_depth_under_pixel(picked_pixel, viewer, viewer->camera());
          if (depth != 1.0)
          {
            //add object to list of picked objects;
@@ -641,13 +665,17 @@ void Scene::renderPointScene(const QList<Scene_interface::Item_id> &items,
          (item.renderingMode() == PointsPlusNormals)  ||
          (item.renderingMode() == ShadedPoints))
       {
+        if(with_names) {
+          viewer->glClearDepthf(1.0);
+          viewer->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+        }
         viewer->setGlPointSize(3.0f);
         item.drawPoints(viewer);
       }
       if(item.renderingMode() == Points && with_names) {
         //    read depth buffer at pick location;
         float depth = 1.0;
-        viewer->glReadPixels(picked_pixel.x(),viewer->camera()->screenHeight()-1-picked_pixel.y(),1,1,GL_DEPTH_COMPONENT, GL_FLOAT, &depth);
+        depth = read_depth_under_pixel(picked_pixel, viewer, viewer->camera());
         if (depth != 1.0)
         {
           //add object to list of picked objects;
@@ -666,7 +694,7 @@ void Scene::renderPointScene(const QList<Scene_interface::Item_id> &items,
        return true;
    return false;
  }
-void 
+void
 Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
 {
     QMap<float, int> picked_item_IDs;
@@ -681,22 +709,37 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
     Q_FOREACH(Item_id id, children)
     {
       Scene_item* item = m_entries[id];
-      if(item->alpha() == 1.0f)
+      Scene_group_item* group = qobject_cast<Scene_group_item*>(item);
+      bool is_transparent=false;
+      if(item->alpha() != 1.0f)
+        is_transparent = true;
+      else if(group)
+      {
+        for(const auto& child : group->getChildren())
+        {
+          if(group->getChild(child)->alpha() < 1.0f)
+          {
+            is_transparent = true;
+            break;
+          }
+        }
+      }
+      if(!is_transparent)
         opaque_items.push_back(id);
       else
         transparent_items.push_back(id);
     }
-    renderScene(children, viewer, picked_item_IDs, with_names, -1, false, NULL);
+    renderScene(children, viewer, picked_item_IDs, with_names, -1, false, nullptr);
     if(with_names)
     {
-      //here we get the selected point, before erasing the depth buffer. We store it 
-      //in a dynamic property as a QList<double>. If there is some alpha, the 
-      //depth buffer is altered, and the picking will return true even when it is 
+      //here we get the selected point, before erasing the depth buffer. We store it
+      //in a dynamic property as a QList<double>. If there is some alpha, the
+      //depth buffer is altered, and the picking will return true even when it is
       // performed in the background, when it should return false. To avoid that,
       // we distinguish the case were there is no alpha, to let the viewer
       //perform it, and the case where the pixel is not found. In the first case,
       //we erase the property, in the latter we return an empty list.
-      //According ot that, in the viewer, either we perform the picking, either we do nothing.
+      //According to that, in the viewer, either we perform the picking, either we do nothing.
       if(has_alpha()) {
         bool found = false;
         CGAL::qglviewer::Vec point = viewer->camera()->pointUnderPixel(picked_pixel, found) - viewer->offset();
@@ -720,12 +763,17 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
       std::vector<QOpenGLFramebufferObject*> fbos;
       std::vector<QOpenGLFramebufferObject*> depth_test;
       QColor background = viewer->backgroundColor();
-   
-      fbos.resize((int)viewer->total_pass());
-      depth_test.resize((int)viewer->total_pass()-1);
-      
+      fbos.resize(static_cast<int>(viewer->total_pass()));
+      depth_test.resize(static_cast<int>(viewer->total_pass())-1);
+
+      int viewport[4];
+      viewer->glGetIntegerv(GL_VIEWPORT, viewport);
+
+      int w = viewport[2];// viewer->width();
+      int h = viewport[3];// viewer->height();
+
       //first pass
-      fbos[0] = new QOpenGLFramebufferObject(viewer->width(), viewer->height(),QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
+      fbos[0] = new QOpenGLFramebufferObject(w, h, QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
       fbos[0]->bind();
       viewer->glDisable(GL_BLEND);
       viewer->glEnable(GL_DEPTH_TEST);
@@ -736,10 +784,11 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
                            0.0f);
       viewer->glClearDepthf(1);
       viewer->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-      renderScene(opaque_items, viewer, picked_item_IDs, false, 0,false, NULL);
-      renderScene(transparent_items, viewer, picked_item_IDs, false, 0,false, NULL);
+      renderScene(opaque_items, viewer, picked_item_IDs, false, 0,false, nullptr);
+      renderScene(transparent_items, viewer, picked_item_IDs, false, 0,false, nullptr);
       fbos[0]->release();
-      depth_test[0] = new QOpenGLFramebufferObject(viewer->width(), viewer->height(),QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
+
+      depth_test[0] = new QOpenGLFramebufferObject(w, h,QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
       depth_test[0]->bind();
       viewer->glDisable(GL_BLEND);
       viewer->glEnable(GL_DEPTH_TEST);
@@ -750,14 +799,14 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
                            0.0f);
       viewer->glClearDepthf(1);
       viewer->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-      renderScene(opaque_items, viewer, picked_item_IDs, false, 0,true, NULL);
-      renderScene(transparent_items, viewer, picked_item_IDs, false, 0,true, NULL);
+      renderScene(opaque_items, viewer, picked_item_IDs, false, 0,true, nullptr);
+      renderScene(transparent_items, viewer, picked_item_IDs, false, 0,true, nullptr);
       depth_test[0]->release();
-   
+
       //other passes
       for(int i=1; i<viewer->total_pass()-1; ++i)
       {
-        fbos[i] = new QOpenGLFramebufferObject(viewer->width(), viewer->height(),QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
+        fbos[i] = new QOpenGLFramebufferObject(w, h,QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
         fbos[i]->bind();
         viewer->glDisable(GL_BLEND);
         viewer->glEnable(GL_DEPTH_TEST);
@@ -773,8 +822,8 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
         renderScene(opaque_items     , viewer, picked_item_IDs, false, i, false, depth_test[i-1]);
         renderScene(transparent_items, viewer, picked_item_IDs, false, i, false, depth_test[i-1]);
         fbos[i]->release();
-   
-        depth_test[i] = new QOpenGLFramebufferObject(viewer->width(), viewer->height(),QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
+
+        depth_test[i] = new QOpenGLFramebufferObject(w, h,QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
         depth_test[i]->bind();
         viewer->glDisable(GL_BLEND);
         viewer->glEnable(GL_DEPTH_TEST);
@@ -789,11 +838,10 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
         renderScene(transparent_items, viewer, picked_item_IDs, false, i, true, depth_test[i-1]);
         depth_test[i]->release();
       }
-   
-   
+
       //last pass
-      fbos[(int)viewer->total_pass()-1] = new QOpenGLFramebufferObject(viewer->width(), viewer->height(),QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
-      fbos[(int)viewer->total_pass()-1]->bind();
+      fbos[static_cast<int>(viewer->total_pass())-1] = new QOpenGLFramebufferObject(w, h,QOpenGLFramebufferObject::Depth, GL_TEXTURE_2D, GL_RGBA32F);
+      fbos[static_cast<int>(viewer->total_pass())-1]->bind();
       viewer->glDisable(GL_BLEND);
       viewer->glEnable(GL_DEPTH_TEST);
       viewer->glDepthFunc(GL_LESS);
@@ -803,31 +851,31 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
                            0.0f);
       viewer->glClearDepthf(1);
       viewer->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-      renderScene(opaque_items     , viewer, picked_item_IDs, false, (int)viewer->total_pass()-1, false, depth_test[(int)viewer->total_pass()-2]);
-      renderScene(transparent_items, viewer, picked_item_IDs, false, (int)viewer->total_pass()-1, false, depth_test[(int)viewer->total_pass()-2]);
-      fbos[(int)viewer->total_pass()-1]->release();
-      if(viewer->getStoredFrameBuffer() != NULL)
+      renderScene(opaque_items     , viewer, picked_item_IDs, false, static_cast<int>(viewer->total_pass())-1, false, depth_test[static_cast<int>(viewer->total_pass())-2]);
+      renderScene(transparent_items, viewer, picked_item_IDs, false, static_cast<int>(viewer->total_pass())-1, false, depth_test[static_cast<int>(viewer->total_pass())-2]);
+      fbos[static_cast<int>(viewer->total_pass())-1]->release();
+      if(viewer->getStoredFrameBuffer() != nullptr)
         viewer->getStoredFrameBuffer()->bind();
-   
+
       //blending
       program.bind();
-      vao->bind();
-      viewer->glClearColor(background.redF(),
-                           background.greenF(),
-                           background.blueF(),
+      vaos[viewer]->bind();
+      viewer->glClearColor(static_cast<GLclampf>(background.redF()),
+                           static_cast<GLclampf>(background.greenF()),
+                           static_cast<GLclampf>(background.blueF()),
                            0.0f);
       viewer->glDisable(GL_DEPTH_TEST);
       viewer->glClear(GL_COLOR_BUFFER_BIT);
       viewer->glEnable(GL_BLEND);
       viewer->glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-      
+
       QMatrix4x4 proj_mat;
       proj_mat.setToIdentity();
       proj_mat.ortho(-1,1,-1,1,0,1);
       program.setUniformValue("projection_matrix", proj_mat);
-      for(int i=0; i< (int)viewer->total_pass()-1; ++i)
+      for(int i=0; i< static_cast<int>(viewer->total_pass())-1; ++i)
         delete depth_test[i];
-      for(int i = (int)viewer->total_pass()-1; i>=0; --i)
+      for(int i = static_cast<int>(viewer->total_pass())-1; i>=0; --i)
       {
         viewer->glBindTexture(GL_TEXTURE_2D, fbos[i]->texture());
         viewer->glDrawArrays(GL_TRIANGLES,0,static_cast<GLsizei>(6));
@@ -835,16 +883,16 @@ Scene::draw_aux(bool with_names, CGAL::Three::Viewer_interface* viewer)
       }
       viewer->glDisable(GL_BLEND);
       viewer->glEnable(GL_DEPTH_TEST);
-      vao->release();
+      vaos[viewer]->release();
       program.release();
     }
-   
+
     viewer->glDepthFunc(GL_LEQUAL);
     // Wireframe OpenGL drawing
     renderWireScene(children, viewer, picked_item_IDs, with_names);
     // Points OpenGL drawing
     renderPointScene(children, viewer, picked_item_IDs, with_names);
-    
+
     if(with_names)
     {
         QList<float> depths = picked_item_IDs.keys();
@@ -1056,11 +1104,13 @@ bool Scene::dropMimeData(const QMimeData * /*data*/,
       CGAL::Three::Scene_group_item* group =
           qobject_cast<CGAL::Three::Scene_group_item*>(item(i));
       if(group)
+      {
         Q_FOREACH(Item_id id, group->getChildren())
         {
           CGAL::Three::Scene_item* child = item(id);
           groups_children << item_id(child);
         }
+      }
     }
     // Insure that children of selected groups will not be added twice
     Q_FOREACH(int i, selected_items_list)
@@ -1071,7 +1121,7 @@ bool Scene::dropMimeData(const QMimeData * /*data*/,
       }
     }
     //Gets the group at the drop position
-    CGAL::Three::Scene_group_item* group = NULL;
+    CGAL::Three::Scene_group_item* group = nullptr;
     if(parent.isValid())
         group = qobject_cast<CGAL::Three::Scene_group_item*>(this->item(index_map[parent]));
     bool one_contained = false;
@@ -1096,7 +1146,7 @@ bool Scene::dropMimeData(const QMimeData * /*data*/,
           if(item->parentGroup())
           {
             item->parentGroup()->removeChild(item);
-            children.push_back(item_id(item));
+            addChild(item);
           }
         }
         redraw_model();
@@ -1112,7 +1162,7 @@ bool Scene::dropMimeData(const QMimeData * /*data*/,
 
 //todo : if a group is selected, don't treat it's children.
 bool Scene::sort_lists(QVector<QList<int> >&sorted_lists, bool up)
-{  
+{
   QVector<int> group_found;
   Q_FOREACH(int i, selectionIndices())
   {
@@ -1135,13 +1185,13 @@ bool Scene::sort_lists(QVector<QList<int> >&sorted_lists, bool up)
       }
     }
   }
-  //iterate the first list to find the groups that are selected and remove the corresponding 
+  //iterate the first list to find the groups that are selected and remove the corresponding
   //sub lists.
   //todo: do that for each group. (treat subgroups)
   for(int i = 0; i< sorted_lists.first().size(); ++i)
   {
     Scene_group_item* group = qobject_cast<Scene_group_item*>(this->item(sorted_lists.first()[i]));
-    if(group && ! group->getChildren().isEmpty())
+    if(group && ! group->getChildren().isEmpty() && sorted_lists.first()[i] < sorted_lists.size())
     {
       sorted_lists[sorted_lists.first()[i]].clear();
     }
@@ -1178,13 +1228,15 @@ bool Scene::sort_lists(QVector<QList<int> >&sorted_lists, bool up)
 }
 void Scene::moveRowUp()
 {
+  if(selectionIndices().isEmpty())
+    return;
   QVector<QList<int> >sorted_lists(1);
   QList<int> to_select;
   //sort lists according to the indices of each item in its container (scene or group)
   //if moving one up would put it out of range, then we stop and do nothing.
   if(!sort_lists(sorted_lists, true))
     return;
-  
+
   for(int i=0; i<sorted_lists.first().size(); ++i)
   {
     Item_id selected_id = sorted_lists.first()[i];
@@ -1229,6 +1281,8 @@ void Scene::moveRowUp()
 }
 void Scene::moveRowDown()
 {
+  if(selectionIndices().isEmpty())
+    return;
   QVector<QList<int> >sorted_lists(1);
   QList<int> to_select;
   //sort lists according to the indices of each item in its container (scene or group)
@@ -1248,7 +1302,7 @@ void Scene::moveRowDown()
         int newId = children.indexOf(
               index_map.value(index(baseId.row()+1, baseId.column(),baseId.parent()))) ;
         children.move(children.indexOf(selected_id), newId);
-      
+
       redraw_model();
       to_select.prepend(m_entries.indexOf(selected_item));
     }
@@ -1278,7 +1332,9 @@ void Scene::moveRowDown()
       }
     }
   }
-  selectionChanged(to_select);
+  if(!to_select.isEmpty()){
+    selectionChanged(to_select);
+  }
 }
 Scene::Item_id Scene::mainSelectionIndex() const {
     return (selectionIndices().size() == 1) ? selected_item : -1;
@@ -1313,12 +1369,18 @@ QItemSelection Scene::createSelection(QList<int> is)
 
 QItemSelection Scene::createSelectionAll()
 {
-  //it is not possible to directly create a selection with items that have different parents, so 
+  //it is not possible to directly create a selection with items that have different parents, so
   //we do it iteratively.
   QItemSelection sel;
-  for(int i=0; i< m_entries.size(); ++i)
-    sel.select(index_map.keys(i).at(0),
-               index_map.keys(i).at(4));
+  sel.select(this->createIndex(0, 0),
+             this->createIndex(m_entries.size(), LastColumn));
+  for(const auto& gid : m_groups)
+  {
+    CGAL::Three::Scene_group_item* group =
+        qobject_cast<CGAL::Three::Scene_group_item*>(item(gid));
+    sel.select(index_map.keys(group->getChildren().first()).at(0),
+               index_map.keys(group->getChildren().last()).at(4));
+  }
   return sel;
 }
 
@@ -1368,7 +1430,7 @@ void Scene::itemVisibilityChanged(CGAL::Three::Scene_item* item)
     //does not recenter
     if(visibility_recentering_enabled){
       Q_EMIT updated_bbox(true);
-      
+
     }
   }
 }
@@ -1404,7 +1466,7 @@ bool SceneDelegate::editorEvent(QEvent *event, QAbstractItemModel *model,
             if(mouseEvent->button() == ::Qt::LeftButton) {
                 QColor color =
                         QColorDialog::getColor(model->data(index).value<QColor>(),
-                                               0/*,
+                                               nullptr/*,
                                                tr("Select color"),
                                                QColorDialog::ShowAlphaChannel*/);
                 if (color.isValid()) {
@@ -1617,18 +1679,18 @@ void Scene::printPrimitiveId(QPoint point, CGAL::Three::Viewer_interface* viewer
       item->printPrimitiveId(point, viewer);
   }
 }
-void Scene::printVertexIds(CGAL::Three::Viewer_interface* viewer)
+void Scene::printVertexIds()
 {
   Scene_item *it = item(mainSelectionIndex());
   if(it)
   {
     Scene_print_item_interface* item= qobject_cast<Scene_print_item_interface*>(it);
     if(item)
-      item->printVertexIds(viewer);
+      item->printVertexIds();
   }
 }
 
-void Scene::printEdgeIds(CGAL::Three::Viewer_interface* viewer)
+void Scene::printEdgeIds()
 {
   Scene_item *it = item(mainSelectionIndex());
   if(it)
@@ -1636,11 +1698,11 @@ void Scene::printEdgeIds(CGAL::Three::Viewer_interface* viewer)
     //Only call printEdgeIds if the item is a Scene_print_item_interface
     Scene_print_item_interface* item= qobject_cast<Scene_print_item_interface*>(it);
     if(item)
-      item->printEdgeIds(viewer);
+      item->printEdgeIds();
   }
 }
 
-void Scene::printFaceIds(CGAL::Three::Viewer_interface* viewer)
+void Scene::printFaceIds()
 {
   Scene_item *it = item(mainSelectionIndex());
   if(it)
@@ -1648,11 +1710,11 @@ void Scene::printFaceIds(CGAL::Three::Viewer_interface* viewer)
     //Only call printFaceIds if the item is a Scene_print_item_interface
     Scene_print_item_interface* item= qobject_cast<Scene_print_item_interface*>(it);
     if(item)
-      item->printFaceIds(viewer);
+      item->printFaceIds();
   }
 }
 
-void Scene::printAllIds(CGAL::Three::Viewer_interface* viewer)
+void Scene::printAllIds()
 {
   Scene_item *it = item(mainSelectionIndex());
   if(it)
@@ -1660,10 +1722,10 @@ void Scene::printAllIds(CGAL::Three::Viewer_interface* viewer)
     //Only call printFaceIds if the item is a Scene_print_item_interface
     Scene_print_item_interface* item= qobject_cast<Scene_print_item_interface*>(it);
     if(item)
-      item->printAllIds(viewer);
+      item->printAllIds();
   }
 }
-void Scene::updatePrimitiveIds(CGAL::Three::Viewer_interface* viewer, CGAL::Three::Scene_item* it)
+void Scene::updatePrimitiveIds(CGAL::Three::Scene_item* it)
 {
   if(it)
   {
@@ -1672,14 +1734,14 @@ void Scene::updatePrimitiveIds(CGAL::Three::Viewer_interface* viewer, CGAL::Thre
     {
       //As this function works as a toggle, the first call hides the ids and the second one shows them again,
       //thereby triggering their re-computation.
-      item->printVertexIds(viewer);
-      item->printVertexIds(viewer);
+      item->printVertexIds();
+      item->printVertexIds();
 
-      item->printEdgeIds(viewer);
-      item->printEdgeIds(viewer);
+      item->printEdgeIds();
+      item->printEdgeIds();
 
-      item->printFaceIds(viewer);
-      item->printFaceIds(viewer);
+      item->printFaceIds();
+      item->printFaceIds();
     }
   }
 }
@@ -1761,7 +1823,6 @@ void Scene::addGroup(Scene_group_item* group)
     connect(this, SIGNAL(drawFinished()), group, SLOT(resetDraw()));
     connect(this, SIGNAL(indexErased(Scene_interface::Item_id)),
                 group, SLOT(adjustIds(Scene_interface::Item_id)));
-    group->setScene(this);
 }
 
 namespace scene { namespace details {
@@ -1772,12 +1833,12 @@ findItem(const CGAL::Three::Scene_interface* scene_interface,
          const QMetaObject& metaobj,
          QString name, Scene_item_name_fn_ptr fn) {
     const Scene* scene = dynamic_cast<const Scene*>(scene_interface);
-    if(!scene) return 0;
+    if(!scene) return nullptr;
     Q_FOREACH(CGAL::Three::Scene_item* item, scene->entries()) {
        CGAL::Three::Scene_item* ptr = qobject_cast<CGAL::Three::Scene_item*>(metaobj.cast(item));
         if(ptr && ((ptr->*fn)() == name)) return ptr;
     }
-    return 0;
+    return nullptr;
 }
 
 Q_DECL_EXPORT
@@ -1830,7 +1891,94 @@ void Scene::adjustIds(Item_id removed_id)
   }
 }
 
+void Scene::computeBbox()
+{
+  if(m_entries.empty())
+  {
+    last_bbox = Bbox(0,0,0,0,0,0);
+    return;
+  }
+
+  bool bbox_initialized = false;
+  Bbox bbox = Bbox(0,0,0,0,0,0);
+  Q_FOREACH(CGAL::Three::Scene_item* item, m_entries)
+  {
+    if(item->isFinite() && !item->isEmpty() ) {
+      if(bbox_initialized) {
+
+        bbox = bbox + item->bbox();
+      }
+      else {
+        bbox = item->bbox();
+        bbox_initialized = true;
+
+      }
+    }
+
+  }
+  last_bbox = bbox;
+}
+
+void Scene::newViewer(Viewer_interface *viewer)
+{
+  initGL(viewer);
+  Q_FOREACH(Scene_item* item, m_entries)
+  {
+    item->newViewer(viewer);
+  }
+}
+
+void Scene::removeViewer(Viewer_interface *viewer)
+{
+ //already destroyed;
+  if(viewer->property("is_destroyed").toBool())
+    return;
+
+  viewer->makeCurrent();
+  vaos[viewer]->destroy();
+  vaos[viewer]->deleteLater();
+  vaos.remove(viewer);
+  Q_FOREACH(Scene_item* item, m_entries)
+  {
+    item->removeViewer(viewer);
+  }
+}
+
+void Scene::initGL(Viewer_interface *viewer)
+{
+  viewer->makeCurrent();
+  vaos[viewer] = new QOpenGLVertexArrayObject();
+  vaos[viewer]->create();
+  program.bind();
+  vaos[viewer]->bind();
+  vbo[0].bind();
+  vbo[0].allocate(points, 18 * sizeof(float));
+  program.enableAttributeArray("vertex");
+  program.setAttributeArray("vertex", GL_FLOAT, nullptr, 3);
+  vbo[0].release();
+
+  vbo[1].bind();
+  vbo[1].allocate(uvs, 12 * sizeof(float));
+  program.enableAttributeArray("v_texCoord");
+  program.setAttributeArray("v_texCoord", GL_FLOAT, nullptr, 2);
+  vbo[1].release();
+  vaos[viewer]->release();
+  program.release();
+}
+
+void Scene::callDraw(){
+  Q_FOREACH(CGAL::QGLViewer* v, CGAL::QGLViewer::QGLViewerPool())
+  {
+    qobject_cast<Viewer_interface*>(v)->update();
+  }
+}
+
 void Scene::enableVisibilityRecentering(bool b)
 {
   visibility_recentering_enabled = b;
 }
+
+void Scene::addChild(Scene_item *item)
+{
+  children.push_back(item_id(item));
+}
