@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5.2/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base_v6.h $
-// $Id: CGAL_Ipelet_base_v6.h 9bf61b7 2020-04-22T11:02:16+02:00 Maxime Gimeno
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base_v6.h $
+// $Id: CGAL_Ipelet_base_v6.h 7a62583efa1 2022-11-14T19:14:33+01:00 albert-github
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -338,12 +338,12 @@ public:
     draw_polyline_in_ipe(const iterator first, const iterator last,
                          bool setclose=false,bool deselect_all=false,
                          bool blackfill=false,
-                         typename boost::enable_if<
-                                    boost::is_same<
+                         std::enable_if_t<
+                                    std::is_same<
                                       typename std::iterator_traits<iterator>::value_type,
                                       Point_2
-                                    >
-                                  >::type* =nullptr) const
+                                    >::value
+                                  >* =nullptr) const
     {
       IpeSegmentSubPath* SSP_ipe=create_polyline(first,last,setclose);
       if (SSP_ipe!=nullptr){
@@ -403,9 +403,9 @@ public:
     {
       IpeSegmentSubPath* SSP_ipe = new IpeSegmentSubPath;
       IpeVector ipeS=IpeVector( CGAL::to_double(std::get<1>(arc).x()),
-                                CGAL::to_double(std::get<1>(arc).y()));//convert ot ipe format
+                                CGAL::to_double(std::get<1>(arc).y()));//convert to ipe format
       IpeVector ipeT=IpeVector( CGAL::to_double(std::get<2>(arc).x()),
-                                CGAL::to_double(std::get<2>(arc).y()));//convert ot ipe format
+                                CGAL::to_double(std::get<2>(arc).y()));//convert to ipe format
       SSP_ipe->AppendArc(IpeMatrix(sqrt(CGAL::to_double(std::get<0>(arc).squared_radius())),0,
                                    0,(std::get<3>(arc)==CGAL::COUNTERCLOCKWISE?1:-1)*
                                      sqrt(CGAL::to_double(std::get<0>(arc).squared_radius())),
@@ -636,13 +636,13 @@ public:
     template<class iterator>
     void
     draw_in_ipe(const iterator begin,const iterator end,const Iso_rectangle_2& bbox,bool make_grp=true,bool deselect_all=false,
-     typename boost::enable_if<  boost::mpl::or_< boost::is_same<typename std::iterator_traits<iterator>::value_type,Point_2> ,
-                                 boost::mpl::or_< boost::is_same<typename std::iterator_traits<iterator>::value_type,Segment_2> ,
-                                 boost::mpl::or_< boost::is_same<typename std::iterator_traits<iterator>::value_type,Circle_2> ,
-                                 boost::mpl::or_< boost::is_same<typename std::iterator_traits<iterator>::value_type,Circular_arc_2> ,
-                                                  boost::is_same<typename std::iterator_traits<iterator>::value_type,Polygon_2>
-                                                > > > >
-                    >::type* = nullptr) const
+     std::enable_if_t<  boost::mpl::or_< std::is_same<typename std::iterator_traits<iterator>::value_type,Point_2> ,
+                        boost::mpl::or_< std::is_same<typename std::iterator_traits<iterator>::value_type,Segment_2> ,
+                        boost::mpl::or_< std::is_same<typename std::iterator_traits<iterator>::value_type,Circle_2> ,
+                        boost::mpl::or_< std::is_same<typename std::iterator_traits<iterator>::value_type,Circular_arc_2> ,
+                                         std::is_same<typename std::iterator_traits<iterator>::value_type,Polygon_2>
+                                      > > > >::value
+                    >* = nullptr) const
     {
       for (iterator it=begin;it!=end;++it)
         draw_in_ipe(*it,bbox);
@@ -945,7 +945,7 @@ public:
               //retrieve circle arcs
               if(SSP_ipe -> Segment(j).Type()==IpePathSegment::EArc &&
                  is_only_rotated_or_scaled(object->AsPath()->Matrix()))
-              {//retreve circle arcs
+              {//retrieve circle arcs
                 if ( !CGAL::Is_in_tuple<Circular_arc_2,typename multi_output_iterator::Value_type_tuple>::value ){
                   to_deselect=true;
                   continue;
