@@ -7,13 +7,16 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5/Number_types/include/CGAL/Exact_rational.h $
-// $Id: Exact_rational.h 98e4718 2021-08-26T11:33:39+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Number_types/include/CGAL/Exact_rational.h $
+// $Id: Exact_rational.h e46196d362c 2023-07-12T13:40:24+02:00 Sébastien Loriot
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Laurent Rineau
 
+#ifndef CGAL_EXACT_RATIONAL_H
+#define CGAL_EXACT_RATIONAL_H
+
 #include <CGAL/Number_types/internal/Exact_type_selector.h>
 
 namespace CGAL {
@@ -39,8 +42,10 @@ typedef unspecified_type Exact_rational;
 
 #else // not DOXYGEN_RUNNING
 
-typedef internal::Exact_field_selector<double>::Type Exact_rational;
+using Exact_rational = internal::Exact_NT_backend<internal::Default_exact_nt_backend>::Rational;
 
 #endif
 
 } /* end namespace CGAL */
+
+#endif // CGAL_EXACT_RATIONAL_H
