@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h $
-// $Id: Hyperbolic_Delaunay_triangulation_CK_traits_2.h 98e4718 2021-08-26T11:33:39+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_CK_traits_2.h $
+// $Id: Hyperbolic_Delaunay_triangulation_CK_traits_2.h b33ce790b87 2024-12-03T09:12:38+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mikhail Bogdanov
@@ -20,7 +20,7 @@
 #include <CGAL/Algebraic_kernel_for_circles_2_2.h>
 #include <CGAL/Circular_kernel_2/Intersection_traits.h>
 #include <CGAL/Circular_kernel_2.h>
-#include <CGAL/triangulation_assertions.h>
+#include <CGAL/assertions.h>
 #include <CGAL/determinant.h>
 
 #include <boost/tuple/tuple.hpp>
@@ -77,13 +77,17 @@ namespace internal {
           std::vector< Intersection_result > inters;
           intersection(*c_pq, *c_qr, std::back_inserter(inters));
 
-          CGAL_triangulation_assertion(assign(pair, inters[0]));
+          CGAL_assertion_code(bool ok=)
+          assign(pair, inters[0]);
+          CGAL_assertion(ok);
           if(pair.second == 1)
           {
             if(_gt.has_on_bounded_side_2_object()(l_inf, pair.first))
               return pair.first;
 
-            CGAL_triangulation_assertion(assign(pair, inters[1]));
+            CGAL_assertion_code(bool ok=)
+            assign(pair, inters[1]);
+            CGAL_assertion(ok);
             return pair.first;
           }
           return pair.first;
@@ -104,13 +108,17 @@ namespace internal {
       std::vector< Intersection_result > inters;
       intersection(*l, *c, std::back_inserter(inters));
 
-      CGAL_triangulation_assertion(assign(pair,inters[0]));
+      CGAL_assertion_code(bool ok=)
+      assign(pair,inters[0]);
+      CGAL_assertion(ok);
       if(pair.second == 1)
       {
         if(_gt.has_on_bounded_side_2_object()(l_inf, pair.first))
           return pair.first;
 
-        CGAL_triangulation_assertion(assign(pair, inters[1]));
+        CGAL_assertion_code(bool ok=)
+        assign(pair, inters[1]);
+        CGAL_assertion(ok);
         return pair.first;
       }
       return pair.first;
@@ -177,9 +185,9 @@ namespace internal {
                                     const Hyperbolic_point_2& r,
                                     const Hyperbolic_point_2& s) const
     {
-      CGAL_triangulation_precondition((_gt.orientation_2_object()(p, q, r) == ON_POSITIVE_SIDE) &&
+      CGAL_precondition((_gt.orientation_2_object()(p, q, r) == ON_POSITIVE_SIDE) &&
                                       (_gt.orientation_2_object()(p, s, q) == ON_POSITIVE_SIDE));
-      CGAL_triangulation_precondition((_gt.side_of_oriented_circle_2_object()(p, q, r,s) == ON_NEGATIVE_SIDE) &&
+      CGAL_precondition((_gt.side_of_oriented_circle_2_object()(p, q, r,s) == ON_NEGATIVE_SIDE) &&
                                       (_gt.side_of_oriented_circle_2_object()(p, s, q, r) == ON_NEGATIVE_SIDE));
 
       Construct_circle_or_line_supporting_bisector<Traits>  cclsb(_gt);
@@ -218,7 +226,7 @@ namespace internal {
                                     const Hyperbolic_point_2& q,
                                     const Hyperbolic_point_2& r) const
     {
-      CGAL_triangulation_precondition(_gt.orientation_2_object()(p, q, r) == POSITIVE);
+      CGAL_precondition(_gt.orientation_2_object()(p, q, r) == POSITIVE);
 
       Construct_circle_or_line_supporting_bisector<Traits>  cclsb(_gt);
       Construct_hyperbolic_circumcenter_CK_2<Traits>        chc(_gt);
@@ -240,13 +248,17 @@ namespace internal {
         intersection(bis_pq, l_inf, std::back_inserter(inters));
         std::pair<Circular_arc_point_2, unsigned> pair;
 
-        CGAL_triangulation_assertion(assign(pair,inters[0]));
-        CGAL_triangulation_assertion(pair.second == 1);
+        CGAL_assertion_code(bool ok=)
+        assign(pair,inters[0]);
+        CGAL_assertion(ok);
+        CGAL_assertion(pair.second == 1);
         if(_gt.less_y_2_object()(p, q))
           return Line_arc_2(bis_pq,a,pair.first);
 
-        CGAL_triangulation_assertion(assign(pair,inters[1]));
-        CGAL_triangulation_assertion(pair.second == 1);
+        CGAL_assertion_code(ok=)
+        assign(pair,inters[1]);
+        CGAL_assertion(ok);
+        CGAL_assertion(pair.second == 1);
         return Line_arc_2(bis_pq,a,pair.first);
       }
 
@@ -261,8 +273,10 @@ namespace internal {
       intersection(*c_pq, l_inf, std::back_inserter(inters));
       std::pair<Circular_arc_point_2, unsigned> pair;
 
-      CGAL_triangulation_assertion(assign(pair,inters[0]));
-      CGAL_triangulation_assertion(pair.second == 1);
+      CGAL_assertion_code(bool ok=)
+      assign(pair,inters[0]);
+      CGAL_assertion(ok);
+      CGAL_assertion(pair.second == 1);
 
       Hyperbolic_point_2 approx_pinf(to_double(pair.first.x()), to_double(pair.first.y()));
       Hyperbolic_point_2 approx_c(to_double(c_pq->center().x()),
@@ -275,7 +289,9 @@ namespace internal {
         return Circular_arc_2(*c_pq, pair.first, a);
       }
 
-      CGAL_triangulation_assertion(assign(pair,inters[1]));
+      CGAL_assertion_code(ok=)
+      assign(pair,inters[1]);
+      CGAL_assertion(ok);
       if(_gt.orientation_2_object()(approx_c,approx_a,approx_pinf) == POSITIVE)
         return Circular_arc_2(*c_pq, pair.first, a);
 
