@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h $
-// $Id: Hyperbolic_Delaunay_triangulation_traits_2.h 98e4718 2021-08-26T11:33:39+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Hyperbolic_triangulation_2/include/CGAL/Hyperbolic_Delaunay_triangulation_traits_2.h $
+// $Id: Hyperbolic_Delaunay_triangulation_traits_2.h d3fca65ae59 2022-09-23T12:49:40+01:00 Andreas Fabri
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Iordan Iordanov
@@ -24,7 +24,7 @@
 #include <CGAL/Hyperbolic_triangulation_2/internal/Exact_complex.h>
 #include <CGAL/Origin.h>
 #include <CGAL/predicates_on_points_2.h>
-#include <CGAL/triangulation_assertions.h>
+#include <CGAL/assertions.h>
 #include <CGAL/utility.h>
 
 #include <boost/tuple/tuple.hpp>
@@ -365,9 +365,9 @@ public:
                                   const Hyperbolic_point_2& r,
                                   const Hyperbolic_point_2& s) const
   {
-    CGAL_triangulation_precondition((_gt.orientation_2_object()(p, q, r) == ON_POSITIVE_SIDE) &&
+    CGAL_precondition((_gt.orientation_2_object()(p, q, r) == ON_POSITIVE_SIDE) &&
                                     (_gt.orientation_2_object()(p, s, q) == ON_POSITIVE_SIDE));
-    CGAL_triangulation_precondition((_gt.side_of_oriented_circle_2_object()(p, q, r,s) == ON_NEGATIVE_SIDE) &&
+    CGAL_precondition((_gt.side_of_oriented_circle_2_object()(p, q, r,s) == ON_NEGATIVE_SIDE) &&
                                     (_gt.side_of_oriented_circle_2_object()(p, s, q, r) == ON_NEGATIVE_SIDE));
 
     Construct_hyperbolic_circumcenter_2<Traits> chc(_gt);
@@ -405,7 +405,7 @@ public:
                                   const Hyperbolic_point_2& q,
                                   const Hyperbolic_point_2& r) const
   {
-    CGAL_triangulation_precondition(_gt.orientation_2_object()(p, q, r) == POSITIVE);
+    CGAL_precondition(_gt.orientation_2_object()(p, q, r) == POSITIVE);
 
     Construct_circle_or_line_supporting_bisector<Traits> cclsb(_gt);
     Construct_hyperbolic_circumcenter_2<Traits> chc(_gt);
