@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.6.1/Point_set_processing_3/include/CGAL/IO/read_las_points.h $
-// $Id: read_las_points.h d3fca65 2022-09-23T12:49:40+01:00 Andreas Fabri
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Point_set_processing_3/include/CGAL/IO/read_las_points.h $
+// $Id: read_las_points.h 98f4633e5f2 2024-09-06T16:07:14+02:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s) : Simon Giraudot
@@ -43,7 +43,7 @@
 #  pragma GCC diagnostic ignored "-Wstrict-aliasing"
 #endif
 
-#define USE_AS_DLL
+#define USE_AS_DLL 1
 #include <lasreader_las.hpp>
 #undef USE_AS_DLL
 
@@ -384,7 +384,11 @@ bool read_LAS_with_properties(std::istream& is,
   if(!is)
     return false;
 
+#if LAS_TOOLS_VERSION < 240319
   LASreaderLAS lasreader;
+#else
+  LASreaderLAS lasreader(nullptr);
+#endif
   lasreader.open(is);
 
   while(lasreader.read_point())
