@@ -2,24 +2,20 @@
 // All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org).
-// You can redistribute it and/or modify it under the terms of the GNU
-// General Public License as published by the Free Software Foundation,
-// either version 3 of the License, or (at your option) any later version.
 //
-// Licensees holding a valid commercial license may use this file in
-// accordance with the commercial license agreement provided with the software.
-//
-// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// $URL: $
-// $Id: $
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Mesh_3/include/CGAL/Mesh_3/Worksharing_data_structures.h $
+// $Id: Worksharing_data_structures.h 8fddaa6baa4 2022-10-03T17:15:27+02:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Clement Jamin
 
 #ifndef CGAL_MESH_3_WORKSHARING_DATA_STRUCTURES_H
 #define CGAL_MESH_3_WORKSHARING_DATA_STRUCTURES_H
 
+#include <CGAL/license/Mesh_3.h>
+
+#include <CGAL/disable_warnings.h>
+
 #ifdef CGAL_LINKED_WITH_TBB
 
 #include <CGAL/Mesh_3/Concurrent_mesher_config.h>
@@ -27,16 +23,14 @@
 #include <CGAL/Bbox_3.h>
 
 #include <tbb/concurrent_queue.h>
-#include <tbb/task.h>
+#include <tbb/task_group.h>
 #include <tbb/enumerable_thread_specific.h>
 #include <tbb/concurrent_vector.h>
 #include <tbb/scalable_allocator.h>
 
+#include <atomic>
 
 #include <vector>
-#ifdef CGAL_MESH_3_TASK_SCHEDULER_SORTED_BATCHES_WITH_MULTISET
-# include <set>
-#endif
 
 namespace CGAL { namespace Mesh_3 {
 
@@ -71,8 +65,8 @@ public:
 
     m_num_cells =
       num_grid_cells_per_axis*num_grid_cells_per_axis*num_grid_cells_per_axis;
-    m_occupation_grid = new tbb::atomic<int>[m_num_cells];
-    m_num_batches_grid = new tbb::atomic<int>[m_num_cells];
+    m_occupation_grid = new std::atomic<int>[m_num_cells];
+    m_num_batches_grid = new std::atomic<int>[m_num_cells];
     // Initialize grid
     for (int i = 0 ; i < m_num_cells ; ++i)
     {
@@ -112,15 +106,15 @@ public:
 
   void add_batch(int cell_index, int to_add)
   {
-    m_num_batches_grid[cell_index].fetch_and_add(to_add);
+    m_num_batches_grid[cell_index].fetch_add(to_add);
   }
 
   void add_occupation(int cell_index, int to_add, int)
   {
-    m_occupation_grid[cell_index].fetch_and_add(to_add);
+    m_occupation_grid[cell_index].fetch_add(to_add);
 
     /*int new_occupation =
-      (m_occupation_grid[cell_index].fetch_and_add(to_add))
+      (m_occupation_grid[cell_index].fetch_add(to_add))
       + to_add;
     //m_num_batches_grid[cell_index] = num_items_in_work_queue;
 
@@ -157,11 +151,11 @@ public:
   {
     // Compute indices on grid
     int index_x = static_cast<int>( (to_double(point.x()) - m_xmin) * m_resolution_x);
-    index_x = std::max( 0, std::min(index_x, m_num_grid_cells_per_axis - 1) );
+    index_x = (std::max)( 0, (std::min)(index_x, m_num_grid_cells_per_axis - 1) );
     int index_y = static_cast<int>( (to_double(point.y()) - m_ymin) * m_resolution_y);
-    index_y = std::max( 0, std::min(index_y, m_num_grid_cells_per_axis - 1) );
+    index_y = (std::max)( 0, (std::min)(index_y, m_num_grid_cells_per_axis - 1) );
     int index_z = static_cast<int>( (to_double(point.z()) - m_zmin) * m_resolution_z);
-    index_z = std::max( 0, std::min(index_z, m_num_grid_cells_per_axis - 1) );
+    index_z = (std::max)( 0, (std::min)(index_z, m_num_grid_cells_per_axis - 1) );
 
     int index =
       index_z*m_num_grid_cells_per_axis*m_num_grid_cells_per_axis
@@ -227,7 +221,7 @@ public:
 
 
     // Rotate
-    static tbb::atomic<int> last_cell_index;
+    static std::atomic<int> last_cell_index;
     //std::cerr << "last=" << last_cell_index << std::endl;
     int i = (last_cell_index + 1) % m_num_cells;
     for ( ; i != last_cell_index ; i = (i + 1) % m_num_cells)
@@ -252,11 +246,11 @@ protected:
 
   int                                             m_num_grid_cells_per_axis;
   int                                             m_num_cells;
-  tbb::atomic<int> *                              m_occupation_grid;
-  tbb::atomic<int> *                              m_num_batches_grid;
+  std::atomic<int> *                              m_occupation_grid;
+  std::atomic<int> *                              m_num_batches_grid;
 
-  tbb::atomic<int>                                m_laziest_cell_index;
-  tbb::atomic<int>                                m_laziest_cell_occupation;
+  std::atomic<int>                                m_laziest_cell_index;
+  std::atomic<int>                                m_laziest_cell_occupation;
 };
 
 
@@ -270,8 +264,10 @@ class WorkItem
 {
 public:
   WorkItem() {}
+  virtual ~WorkItem() { }
+
   // Derived class defines the actual work.
-  virtual void run() = 0;
+  virtual void operator()() const = 0;
   virtual bool less_than(const WorkItem &) const = 0;
 };
 
@@ -298,13 +294,17 @@ public:
     : m_func(func), m_quality(quality)
   {}
 
-  void run()
+  ~MeshRefinementWorkItem() override
+  {}
+
+  void operator()() const override
   {
     m_func();
-    tbb::scalable_allocator<MeshRefinementWorkItem<Func, Quality> >().deallocate(this, 1);
+    tbb::scalable_allocator<MeshRefinementWorkItem>().deallocate(
+        const_cast<MeshRefinementWorkItem *>(this), 1);
   }
-  
-  bool less_than (const WorkItem &other) const
+
+  bool less_than (const WorkItem &other) const override
   {
     /*try
     {
@@ -342,17 +342,20 @@ public:
     : m_func(func)
   {}
 
-  void run()
+  ~SimpleFunctorWorkItem() override = default;
+
+  void operator()() const override
   {
     m_func();
-    tbb::scalable_allocator<SimpleFunctorWorkItem<Func> >().deallocate(this, 1);
+    tbb::scalable_allocator<SimpleFunctorWorkItem>().deallocate(
+        const_cast<SimpleFunctorWorkItem *>(this), 1);
   }
-  
+
   // Irrelevant here
-  bool less_than (const WorkItem &other) const 
-  { 
+  bool less_than (const WorkItem &other) const override
+  {
     // Just compare addresses
-    return this < &other; 
+    return this < &other;
   }
 
 private:
@@ -368,46 +371,24 @@ private:
 class WorkBatch
 {
 public:
-
-#ifdef CGAL_MESH_3_TASK_SCHEDULER_SORTED_BATCHES_WITH_MULTISET
-  typedef std::multiset<WorkItem *, CompareTwoWorkItems> Batch;
-#else
   typedef std::vector<WorkItem *> Batch;
-#endif
-  typedef Batch::const_iterator BatchConstIterator;
-  typedef Batch::iterator       BatchIterator;
+  typedef Batch::const_iterator   BatchConstIterator;
+  typedef Batch::iterator         BatchIterator;
 
   WorkBatch() {}
 
   void add_work_item(WorkItem *p_item)
   {
-#ifdef CGAL_MESH_3_TASK_SCHEDULER_SORTED_BATCHES_WITH_MULTISET
-    m_batch.insert(p_item);
-#else
     m_batch.push_back(p_item);
-#endif
   }
 
-  void move_first_elements(WorkBatch &dest, int num_elements)
+  void operator()() const
   {
-    BatchIterator it = m_batch.begin();
-    BatchIterator it_end = m_batch.end();
-    for (int i = 0 ; i < num_elements && it != it_end ; ++it)
-    {
-      dest.add_work_item(*it);
-    }
-    m_batch.erase(m_batch.begin(), it);
-  }
-
-  void run()
-  {
-#ifdef CGAL_MESH_3_TASK_SCHEDULER_SORTED_BATCHES_WITH_SORT
     std::sort(m_batch.begin(), m_batch.end(), CompareTwoWorkItems());
-#endif
     BatchIterator it = m_batch.begin();
     BatchIterator it_end = m_batch.end();
     for ( ; it != it_end ; ++it)
-      (*it)->run();
+      (*it)->operator()();
   }
 
   size_t size() const
@@ -421,7 +402,7 @@ public:
   }
 
 protected:
-  Batch m_batch;
+  mutable Batch m_batch;
 };
 
 
@@ -433,7 +414,6 @@ protected:
  * ===================
  */
 class WorkItemTask
-  : public tbb::task
 {
 public:
   WorkItemTask(WorkItem *pwi)
@@ -442,7 +422,7 @@ public:
   }
 
 private:
-  /*override*/inline tbb::task* execute();
+  inline void operator()() const;
 
   WorkItem *m_pwi;
 };
@@ -455,57 +435,27 @@ private:
  */
 class Simple_worksharing_ds
 {
-public:
-  // Constructors
-  Simple_worksharing_ds()
-  {
-  }
-
-  /// Destructor
-  virtual ~Simple_worksharing_ds()
-  {
-  }
-
   template <typename Func>
-  void enqueue_work(Func f, tbb::task &parent_task) const
+  void enqueue_work(Func f, tbb::task_group &task_group) const
   {
-    //WorkItem *p_item = new SimpleFunctorWorkItem<Func>(f);
-    WorkItem *p_item = 
-      tbb::scalable_allocator<SimpleFunctorWorkItem<Func> >().allocate(1);
-    new (p_item) SimpleFunctorWorkItem<Func>(f);
-    enqueue_task(create_task(p_item, parent_task));
-  }
-  
-protected:
-  
-  WorkItemTask *create_task(WorkItem *pwi, tbb::task &parent_task) const
-  {
-    return new(tbb::task::allocate_additional_child_of(parent_task)) WorkItemTask(pwi);
-  }
-
-  void enqueue_task(WorkItemTask *t) const
-  {
-    tbb::task::spawn(*t);
+    task_group.run(f);
   }
 };
 
-
-
 /*
  * ==================
  * class TokenTask
  * ==================
  */
 class TokenTask
-  : public tbb::task
 {
 public:
   TokenTask(Load_based_worksharing_ds *p_wsds)
     : m_worksharing_ds(p_wsds) {}
 
-private:
-  /*override*/inline tbb::task* execute();
+  inline void operator()() const;
 
+private:
   Load_based_worksharing_ds *m_worksharing_ds;
 };
 
@@ -528,7 +478,7 @@ public:
   {
     m_tls_work_buffers = new TLS_WorkBuffer[m_num_cells];
     m_work_batches = new tbb::concurrent_queue<WorkBatch>[m_num_cells];
-    m_num_batches = new tbb::atomic<int>[m_num_cells];
+    m_num_batches = new std::atomic<int>[m_num_cells];
 
     for (int i = 0 ; i < m_num_cells ; ++i)
       m_num_batches[i] = 0;
@@ -550,7 +500,7 @@ public:
   }
 
   template <typename P3, typename Func, typename Quality>
-  void enqueue_work(Func f, const Quality &quality, tbb::task &parent_task, const P3 &point)
+  void enqueue_work(Func f, const Quality &quality, tbb::task_group &task_group, const P3 &point)
   {
     WorkItem *p_item = new MeshRefinementWorkItem<Func, Quality>(f, quality);
     int index = m_stats.compute_index(point);
@@ -558,13 +508,13 @@ public:
     wb.add_work_item(p_item);
     if (wb.size() >= NUM_WORK_ITEMS_PER_BATCH)
     {
-      add_batch_and_enqueue_task(wb, index, parent_task);
+      add_batch_and_enqueue_task(wb, index, task_group);
       wb.clear();
     }
   }
 
   // Returns true if some items were flushed
-  bool flush_work_buffers(tbb::task &parent_task)
+  bool flush_work_buffers(tbb::task_group &task_group)
   {
     int num_flushed_items = 0;
 
@@ -584,7 +534,7 @@ public:
     }
 
     for (int i = 0 ; i < num_flushed_items ; ++i)
-      enqueue_task(parent_task);
+      enqueue_task(task_group);
 
     return (num_flushed_items > 0);
   }
@@ -617,7 +567,7 @@ public:
     std::cerr << "Running a batch of " << wb.size() <<
       " elements on cell #" << index << std::endl;
 #endif
-    wb.run();
+    wb();
     add_occupation(index, -1);
   }
 
@@ -634,19 +584,19 @@ protected:
     m_stats.add_batch(index, 1);
   }
 
-  void enqueue_task(tbb::task &parent_task)
+  void enqueue_task(tbb::task_group &task_group)
   {
-    parent_task.increment_ref_count();
     // Warning: when using "enqueue", the system will use up to two threads
     // even if you told task_scheduler_init to use only one
     // (see http://software.intel.com/en-us/forums/showthread.php?t=101669)
-    tbb::task::spawn(*new(parent_task.allocate_child()) TokenTask(this));
+    task_group.run(TokenTask(this));
   }
 
-  void add_batch_and_enqueue_task(const WorkBuffer &wb, int index, tbb::task &parent_task)
+  void add_batch_and_enqueue_task(const WorkBuffer &wb, int index,
+                                  tbb::task_group &task_group)
   {
     add_batch(wb, index);
-    enqueue_task(parent_task);
+    enqueue_task(task_group);
   }
 
   void add_occupation(int cell_index, int to_add, int occupation_radius = 1)
@@ -661,16 +611,16 @@ protected:
     int index_x = cell_index;
 
     // For each cell inside the square
-    for (int i = std::max(0, index_x-occupation_radius) ;
-          i <= std::min(m_num_cells_per_axis - 1, index_x+occupation_radius) ;
+    for (int i = (std::max)(0, index_x-occupation_radius) ;
+          i <= (std::min)(m_num_cells_per_axis - 1, index_x+occupation_radius) ;
           ++i)
     {
-      for (int j = std::max(0, index_y-occupation_radius) ;
-            j <= std::min(m_num_cells_per_axis - 1, index_y+occupation_radius) ;
+      for (int j = (std::max)(0, index_y-occupation_radius) ;
+            j <= (std::min)(m_num_cells_per_axis - 1, index_y+occupation_radius) ;
             ++j)
       {
-        for (int k = std::max(0, index_z-occupation_radius) ;
-              k <= std::min(m_num_cells_per_axis - 1, index_z+occupation_radius) ;
+        for (int k = (std::max)(0, index_z-occupation_radius) ;
+              k <= (std::min)(m_num_cells_per_axis - 1, index_z+occupation_radius) ;
               ++k)
         {
           int index =
@@ -697,7 +647,7 @@ protected:
   Work_statistics                   m_stats;
   TLS_WorkBuffer                   *m_tls_work_buffers;
   tbb::concurrent_queue<WorkBatch> *m_work_batches;
-  tbb::atomic<int>                 *m_num_batches;
+  std::atomic<int>                 *m_num_batches;
 };
 
 
@@ -713,18 +663,16 @@ protected:
  * ===================
  */
 class WorkBatchTask
-  : public tbb::task
 {
 public:
   WorkBatchTask(const WorkBatch &wb)
     : m_wb(wb)
   {
-    //set_affinity(tbb::task::self().affinity());
   }
 
-private:
-  /*override*/inline tbb::task* execute();
+  inline void operator()() const;
 
+private:
   WorkBatch m_wb;
 };
 
@@ -753,31 +701,27 @@ public:
   {
     // We don't need it.
   }
-  
+
   template <typename Func>
-  void enqueue_work(Func f, tbb::task &parent_task)
+  void enqueue_work(Func f, tbb::task_group &task_group)
   {
     //WorkItem *p_item = new SimpleFunctorWorkItem<Func>(f);
-    WorkItem *p_item = 
+    WorkItem *p_item =
       tbb::scalable_allocator<SimpleFunctorWorkItem<Func> >().allocate(1);
     new (p_item) SimpleFunctorWorkItem<Func>(f);
     WorkBatch &workbuffer = m_tls_work_buffers.local();
     workbuffer.add_work_item(p_item);
     if (workbuffer.size() >= NUM_WORK_ITEMS_PER_BATCH)
     {
-      /*WorkBatch wb;
-      workbuffer.move_first_elements(wb, NUM_WORK_ITEMS_PER_BATCH);
-      add_batch_and_enqueue_task(wb, parent_task);*/
-      add_batch_and_enqueue_task(workbuffer, parent_task);
+      add_batch_and_enqueue_task(workbuffer, task_group);
       workbuffer.clear();
     }
   }
 
   template <typename Func, typename Quality>
-  void enqueue_work(Func f, const Quality &quality, tbb::task &parent_task)
+  void enqueue_work(Func f, const Quality &quality, tbb::task_group &task_group)
   {
-    //WorkItem *p_item = new MeshRefinementWorkItem<Func, Quality>(f, quality);
-    WorkItem *p_item = 
+    WorkItem *p_item =
       tbb::scalable_allocator<MeshRefinementWorkItem<Func, Quality> >()
       .allocate(1);
     new (p_item) MeshRefinementWorkItem<Func, Quality>(f, quality);
@@ -785,20 +729,18 @@ public:
     workbuffer.add_work_item(p_item);
     if (workbuffer.size() >= NUM_WORK_ITEMS_PER_BATCH)
     {
-      /*WorkBatch wb;
-      workbuffer.move_first_elements(wb, NUM_WORK_ITEMS_PER_BATCH);
-      add_batch_and_enqueue_task(wb, parent_task);*/
-      add_batch_and_enqueue_task(workbuffer, parent_task);
+      add_batch_and_enqueue_task(workbuffer, task_group);
       workbuffer.clear();
     }
   }
 
   // Returns true if some items were flushed
-  bool flush_work_buffers(tbb::task &parent_task)
+  bool flush_work_buffers(tbb::task_group &task_group)
   {
     int num_flushed_items = 0;
 
-    std::vector<WorkBatchTask*> tasks;
+    std::vector<WorkBatchTask> tasks;
+    tasks.reserve(m_tls_work_buffers.size());
 
     for (TLS_WorkBuffer::iterator it_buffer = m_tls_work_buffers.begin() ;
           it_buffer != m_tls_work_buffers.end() ;
@@ -806,67 +748,68 @@ public:
     {
       if (it_buffer->size() > 0)
       {
-        tasks.push_back(create_task(*it_buffer, parent_task));
+        tasks.push_back(create_task(*it_buffer));
         it_buffer->clear();
         ++num_flushed_items;
       }
     }
 
-    for (std::vector<WorkBatchTask*>::const_iterator it = tasks.begin() ;
+    for (auto it = tasks.begin() ;
       it != tasks.end() ; ++it)
     {
-      enqueue_task(*it, parent_task);
+      enqueue_task(*it, task_group);
     }
 
     return (num_flushed_items > 0);
   }
 
+  [[deprecated]] int approximate_number_of_enqueued_element() const {
+    return 0;
+  }
+
 protected:
 
   // TLS
   typedef WorkBatch                                        WorkBuffer;
   typedef tbb::enumerable_thread_specific<WorkBuffer>      TLS_WorkBuffer;
 
-  WorkBatchTask *create_task(const WorkBuffer &wb, tbb::task &parent_task) const
+  WorkBatchTask create_task(const WorkBuffer &wb) const
   {
-    return new(tbb::task::allocate_additional_child_of(parent_task)) WorkBatchTask(wb);
+    return { wb };
   }
 
-  void enqueue_task(WorkBatchTask *task,
-                    tbb::task &) const
+  void enqueue_task(const WorkBatchTask& task,
+                    tbb::task_group &task_group) const
   {
-    tbb::task::spawn(*task);
+    task_group.run(task);
   }
 
   void add_batch_and_enqueue_task(const WorkBuffer &wb,
-                                  tbb::task &parent_task) const
+                                  tbb::task_group &task_group) const
   {
-    enqueue_task(create_task(wb, parent_task), parent_task);
+    enqueue_task(create_task(wb), task_group);
   }
 
-  const int                         NUM_WORK_ITEMS_PER_BATCH;
+  const size_t                      NUM_WORK_ITEMS_PER_BATCH;
   TLS_WorkBuffer                    m_tls_work_buffers;
 };
 
 
 
 
-inline tbb::task* TokenTask::execute()
+inline void TokenTask::operator()() const
 {
   m_worksharing_ds->run_next_work_item();
-  return NULL;
 }
 
-inline tbb::task* WorkItemTask::execute()
+inline void WorkItemTask::operator()() const
 {
-  m_pwi->run();
-  return NULL;
+  m_pwi->operator()();
 }
 
-inline tbb::task* WorkBatchTask::execute()
+inline void WorkBatchTask::operator()() const
 {
-  m_wb.run();
-  return NULL;
+  m_wb.operator()();
 }
 
 } } //namespace CGAL::Mesh_3
@@ -879,4 +822,6 @@ namespace CGAL { namespace Mesh_3 {
 
 #endif // CGAL_LINKED_WITH_TBB
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_MESH_3_WORKSHARING_DATA_STRUCTURES_H
