@@ -2,21 +2,21 @@
 // INRIA Saclay - Ile de France (France).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 3 of the License,
-// or (at your option) any later version.
+// This file is part of CGAL (www.cgal.org)
 //
-// Licensees holding a valid commercial license may use this file in
-// accordance with the commercial license agreement provided with the software.
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Number_types/include/CGAL/Mpzf.h $
+// $Id: Mpzf.h f3d0d76f6b2 2022-10-06T15:35:41+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
-// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// Author(s)	:  Marc Glisse
+// Author(s)        :  Marc Glisse
 
 #ifndef CGAL_MPZF_H
 #define CGAL_MPZF_H
+
+#ifndef CGAL_NO_MPZF_DIVISION_OPERATOR
+#define CGAL_MPZF_DIVISION_OPERATOR 1
+#endif
+
 #include <cstdlib>
 #include <algorithm>
 #include <climits>
@@ -54,6 +54,10 @@
 #ifndef mpn_neg
 #define mpn_neg mpn_neg_n
 #endif
+// GMP-4.3.0 is missing mpn_sqr.
+#ifndef mpn_sqr
+#define mpn_sqr(dest,a,n) mpn_mul_n(dest,a,a,n)
+#endif
 // GMP before 5.0 doesn't provide mpn_copyi.
 #ifndef mpn_copyi
 #define mpn_copyi(dst, src, siz) std::copy((src), (src)+(siz), (dst))
@@ -71,20 +75,21 @@
 #include <builtins.h>
 #endif
 
+#include <CGAL/assertions.h>
+#include <boost/config.hpp>
+#include <boost/detail/workaround.hpp>
+#include <boost/version.hpp>
+
 #if defined(BOOST_MSVC)
 #  pragma warning(push)
-#  pragma warning(disable:4146 4244 4267 4800)
+#  pragma warning(disable:4146 4244 4267 4702 4800)
      // warning on - applied on unsigned number
      // conversion with loss of data
      // conversion with loss of data
+     // unreachable code
      // int to bool performance
 #endif
 
-#if defined(__GNUC__) && defined(__GNUC_MINOR__) \
-    && (__GNUC__ * 100 + __GNUC_MINOR__) >= 408 \
-    && __cplusplus >= 201103L
-#define CGAL_CAN_USE_CXX11_THREAD_LOCAL
-#endif
 
 /*
 #ifdef CGAL_MPZF_NO_USE_CACHE
@@ -108,16 +113,9 @@
 #if !defined(CGAL_HAS_THREADS)
 #define CGAL_MPZF_THREAD_LOCAL
 #define CGAL_MPZF_TLS
-#elif defined(CGAL_CAN_USE_CXX11_THREAD_LOCAL)
+#else
 #define CGAL_MPZF_THREAD_LOCAL thread_local
 #define CGAL_MPZF_TLS thread_local
-#elif defined(_MSC_VER)
-#define CGAL_MPZF_THREAD_LOCAL __declspec(thread)
-#define CGAL_MPZF_TLS
-#else
-#define CGAL_MPZF_THREAD_LOCAL __thread
-#define CGAL_MPZF_TLS
-// Too bad for the others
 #endif
 namespace CGAL {
 namespace Mpzf_impl {
@@ -137,7 +135,7 @@ template <class T, class = void> struct pool1 {
 template <class T, class D> std::vector<T> pool1<T,D>::data;
 
 // Use an intrusive single-linked list instead (allocate one more limb and use
-// it to store the pointer to next), the difference isn't that noticable (still
+// it to store the pointer to next), the difference isn't that noticeable (still
 // the list wins).  Neither is thread-safe (both can be with threadlocal, and
 // the list can be with an atomic compare-exchange (never tried)).  With gcc,
 // TLS has a large effect on classes with constructor/destructor, but is free
@@ -150,7 +148,7 @@ template <class T, class = void> struct pool2 {
   static bool empty() { return data() == 0; }
   static const int extra = 1; // TODO: handle the case where a pointer is larger than a mp_limb_t
   private:
-  BOOST_STATIC_ASSERT(sizeof(T) >= sizeof(T*));
+  CGAL_static_assertion(sizeof(T) >= sizeof(T*));
   static T& data () {
     static CGAL_MPZF_TLS T data_ = 0;
     return data_;
@@ -158,21 +156,20 @@ template <class T, class = void> struct pool2 {
   static T& ptr(T t) { t -= extra+1; return *reinterpret_cast<T*>(t); }
 };
 
-#if defined(CGAL_CAN_USE_CXX11_THREAD_LOCAL)
 template <class T, class = void> struct pool3 {
   static T pop() { T ret = data(); data() = ptr(data()); return ret; }
   static void push(T t) { ptr(t) = data(); data() = t; }
   static bool empty() { return data() == 0; }
   static const int extra = 1; // TODO: handle the case where a pointer is larger than a mp_limb_t
   private:
-  BOOST_STATIC_ASSERT(sizeof(T) >= sizeof(T*));
+  CGAL_static_assertion(sizeof(T) >= sizeof(T*));
   struct cleaner {
     T data_ = 0;
     ~cleaner(){
       // Deallocate everything. As an alternative, we could store it in a
       // global location, for re-use by a later thread.
       while (!empty())
-	delete[] (pop() - (extra + 1));
+        delete[] (pop() - (extra + 1));
     }
   };
   static T& data () {
@@ -181,7 +178,6 @@ template <class T, class = void> struct pool3 {
   }
   static T& ptr(T t) { t -= extra+1; return *reinterpret_cast<T*>(t); }
 };
-#endif
 
 // No caching
 template <class T, class = void> struct no_pool {
@@ -220,15 +216,7 @@ inline int clz (boost::uint64_t x) {
 // In C++11, std::fill_n returns a pointer to the end, but in C++03,
 // it returns void.
 inline mp_limb_t* fill_n_ptr(mp_limb_t* p, int n, int c) {
-#if __cplusplus >= 201103L
   return std::fill_n (p, n, c);
-#else
-  mp_limb_t* q = p + n;
-  std::fill (p, q, c);
-  //std::fill_n (p, n, c);
-  //memset (p, sizeof(mp_limb_t)*n, c);
-  return q;
-#endif
 }
 } // namespace Mpzf_impl
 
@@ -262,14 +250,16 @@ struct Mpzf {
 //#endif
 
   mp_limb_t* data_; /* data_[0] is never 0 (except possibly for 0). */
-  inline mp_limb_t*& data() { return data_; };
-  inline mp_limb_t const* data() const { return data_; };
+  inline mp_limb_t*& data() { return data_; }
+  inline mp_limb_t const* data() const { return data_; }
 
 #ifdef CGAL_MPZF_USE_CACHE
   mp_limb_t cache[cache_size + 1];
 #endif
   int size; /* Number of relevant limbs in data_. */
   int exp; /* The number is data_ (an integer) * 2 ^ (64 * exp). */
+  typedef int Exponent_type;
+  typedef int Size_type;
 
   struct allocate{};
   struct noalloc{};
@@ -287,12 +277,20 @@ struct Mpzf {
       if(data()[-1] >= mini) return; // TODO: when mini==2, no need to check
       delete[] (data() - (pool::extra+1)); // too small, useless
     }
+#ifndef CGAL_MPZF_USE_CACHE
     if(mini<2) mini=2;
+#endif
     data() = (new mp_limb_t[mini+(pool::extra+1)]) + (pool::extra+1);
     data()[-1] = mini;
   }
   void clear(){
-    while(*--data()==0); // in case we skipped final zeroes
+    // while(*--data()==0);
+    // This line gave a misscompilation by Intel Compiler 2019
+    // (19.0.0.117). I replaced it by the following two lines:
+    // -- Laurent Rineau, sept. 2018
+    --data();
+    while(*data()==0) { --data(); } // in case we skipped final zeroes
+
 #ifdef CGAL_MPZF_USE_CACHE
     if (data() == cache) return;
 #endif
@@ -325,7 +323,7 @@ struct Mpzf {
 #ifdef CGAL_MPZF_USE_CACHE
       if (data() != cache)
 #endif
-	delete[] (data() - pool::extra);
+        delete[] (data() - pool::extra);
       init(asize);
     } else ++data();
     size=x.size;
@@ -340,19 +338,62 @@ struct Mpzf {
     exp=x.exp;
     if(size!=0) mpn_copyi(data(),x.data(),asize);
   }
-#if !defined(CGAL_CFG_NO_CPP0X_RVALUE_REFERENCE) \
-    && !defined(CGAL_MPZF_USE_CACHE)
+#if defined(CGAL_MPZF_USE_CACHE)
+  Mpzf(Mpzf&& x)noexcept:size(x.size),exp(x.exp){
+    auto xd = x.data();
+    while(*--xd==0);
+    if (xd != x.cache) {
+      data() = x.data();
+      x.init();
+    } else {
+      init();
+      if(size!=0) mpn_copyi(data(),x.data(),std::abs(size));
+    }
+    x.size = 0;
+  }
+  Mpzf& operator=(Mpzf&& x)noexcept{
+    if (this == &x) return *this; // is this needed?
+    size = x.size;
+    exp = x.exp;
+    auto xd = x.data();
+    auto td = data();
+    while(*--xd==0);
+    while(*--td==0);
+    if (xd != x.cache) {
+      data() = x.data();
+      if (td != cache) {
+        pool::push(td+1);
+        // should we instead give it to x in case x is reused?
+        // x.data() = td + 1;
+      }
+      x.init();
+    } else {
+      // In some cases data points in the middle of the buffer, reset it
+      data() = td + 1;
+      if(size!=0) mpn_copyi(data(),x.data(),std::abs(size));
+    }
+    x.size = 0;
+    return *this;
+  }
+#else
   Mpzf(Mpzf&& x):data_(x.data()),size(x.size),exp(x.exp){
     x.init(); // yes, that's a shame...
     x.size = 0;
     x.exp = 0;
   }
-  Mpzf& operator=(Mpzf&& x){
-    std::swap(size,x.size);
+  Mpzf& operator=(Mpzf&& x)noexcept{
+    size = x.size;
+    // In case something tries to read it, size needs to be smaller than data
+    x.size = 0;
     exp = x.exp;
     std::swap(data(),x.data());
     return *this;
   }
+  friend void swap(Mpzf&a, Mpzf&b)noexcept{
+    std::swap(a.size, b.size);
+    std::swap(a.exp, b.exp);
+    std::swap(a.data(), b.data());
+  }
   friend Mpzf operator-(Mpzf&& x){
     Mpzf ret = std::move(x);
     ret.size = -ret.size;
@@ -406,15 +447,15 @@ struct Mpzf {
     if (dexp == 0) {
       if (d == 0) { size=0; exp=0; return; }
       else { // denormal number
-	m = u.s.man;
-	++dexp;
+        m = u.s.man;
+        ++dexp;
       }
     } else {
       m = (1LL<<52) | u.s.man;
     }
     int e1 = (int)dexp+13;
     // FIXME: make it more general! But not slower...
-    BOOST_STATIC_ASSERT(GMP_NUMB_BITS == 64);
+    CGAL_static_assertion(GMP_NUMB_BITS == 64);
     int e2 = e1 % 64;
     exp = e1 / 64 - 17;
     // 52+1023+13==17*64 ?
@@ -427,10 +468,10 @@ struct Mpzf {
     }else{
       data()[0] = m << e2;
       if(e2>11){ // Wrong test for denormals
-	data()[1] = m >> (64-e2);
-	size = 2;
+        data()[1] = m >> (64-e2);
+        size = 2;
       } else {
-	size = 1;
+        size = 1;
       }
     }
 #else
@@ -446,16 +487,16 @@ struct Mpzf {
     else {
       data()[0] = d0;
       if (d1 == 0) {
-	size = 1;
+        size = 1;
       }
       else {
-	data()[1] = d1;
-	size = 2;
+        data()[1] = d1;
+        size = 2;
       }
     }
 #endif
     if(u.s.sig) size=-size;
-    //assert(to_double()==IA_force_to_double(d));
+    //CGAL_assertion(to_double()==IA_force_to_double(d));
   }
 
 #ifdef CGAL_USE_GMPXX
@@ -467,8 +508,8 @@ struct Mpzf {
     init_from_mpz_t(z.mpz());
   }
   void init_from_mpz_t(mpz_t const z){
-    exp=mpz_scan1(z,0)/GMP_NUMB_BITS;
-    size=mpz_size(z)-exp;
+    exp=Exponent_type(mpz_scan1(z,0)/GMP_NUMB_BITS);
+    size=Size_type(mpz_size(z)-exp);
     init(size);
     mpn_copyi(data(),z->_mp_d+exp,size);
   }
@@ -535,6 +576,12 @@ struct Mpzf {
   friend bool operator!=(Mpzf const&a, Mpzf const&b){
     return !(a==b);
   }
+  friend Mpzf const& min BOOST_PREVENT_MACRO_SUBSTITUTION (Mpzf const&a, Mpzf const&b){
+    return (b<a)?b:a;
+  }
+  friend Mpzf const& max BOOST_PREVENT_MACRO_SUBSTITUTION (Mpzf const&a, Mpzf const&b){
+    return (a<b)?b:a;
+  }
   private:
   static Mpzf aors(Mpzf const&a, Mpzf const&b, int bsize){
     Mpzf res=noalloc();
@@ -570,53 +617,53 @@ struct Mpzf {
       res.size=0;
       // TODO: if aexp>0, swap a and b so we don't repeat the code.
       if(0<bexp){
-	if(absasize<=bexp){ // no overlap
-	  mpn_copyi(rdata, adata, absasize);
-	  rdata+=absasize;
-	  rdata=Mpzf_impl::fill_n_ptr(rdata,bexp-absasize,0);
-	  mpn_copyi(rdata, bdata, absbsize);
-	  res.size=absbsize+bexp;
-	  if(bsize<0) res.size=-res.size;
-	  return res;
-	} else {
-	  mpn_copyi(rdata, adata, bexp);
-	  adata+=bexp;
-	  absasize-=bexp;
-	  rdata+=bexp;
-	  res.size=bexp;
-	}
+        if(absasize<=bexp){ // no overlap
+          mpn_copyi(rdata, adata, absasize);
+          rdata+=absasize;
+          rdata=Mpzf_impl::fill_n_ptr(rdata,bexp-absasize,0);
+          mpn_copyi(rdata, bdata, absbsize);
+          res.size=absbsize+bexp;
+          if(bsize<0) res.size=-res.size;
+          return res;
+        } else {
+          mpn_copyi(rdata, adata, bexp);
+          adata+=bexp;
+          absasize-=bexp;
+          rdata+=bexp;
+          res.size=bexp;
+        }
       }
       else if(0<aexp){
-	if(absbsize<=aexp){ // no overlap
-	  mpn_copyi(rdata, bdata, absbsize);
-	  rdata+=absbsize;
-	  rdata=Mpzf_impl::fill_n_ptr(rdata,aexp-absbsize,0);
-	  mpn_copyi(rdata, adata, absasize);
-	  res.size=absasize+aexp;
-	  if(asize<0) res.size=-res.size;
-	  return res;
-	} else {
-	  mpn_copyi(rdata, bdata, aexp);
-	  bdata+=aexp;
-	  absbsize-=aexp;
-	  rdata+=aexp;
-	  res.size=aexp;
-	}
+        if(absbsize<=aexp){ // no overlap
+          mpn_copyi(rdata, bdata, absbsize);
+          rdata+=absbsize;
+          rdata=Mpzf_impl::fill_n_ptr(rdata,aexp-absbsize,0);
+          mpn_copyi(rdata, adata, absasize);
+          res.size=absasize+aexp;
+          if(asize<0) res.size=-res.size;
+          return res;
+        } else {
+          mpn_copyi(rdata, bdata, aexp);
+          bdata+=aexp;
+          absbsize-=aexp;
+          rdata+=aexp;
+          res.size=aexp;
+        }
       }
       if(absasize>=absbsize){
-	mp_limb_t carry=mpn_add(rdata,adata,absasize,bdata,absbsize);
-	res.size+=absasize;
-	if(carry!=0){
-	  res.size++;
-	  rdata[absasize]=carry;
-	}
+        mp_limb_t carry=mpn_add(rdata,adata,absasize,bdata,absbsize);
+        res.size+=absasize;
+        if(carry!=0){
+          res.size++;
+          rdata[absasize]=carry;
+        }
       } else {
-	mp_limb_t carry=mpn_add(rdata,bdata,absbsize,adata,absasize);
-	res.size+=absbsize;
-	if(carry!=0){
-	  res.size++;
-	  rdata[absbsize]=carry;
-	}
+        mp_limb_t carry=mpn_add(rdata,bdata,absbsize,adata,absasize);
+        res.size+=absbsize;
+        if(carry!=0){
+          res.size++;
+          rdata[absbsize]=carry;
+        }
       }
       // unnecessary if a.exp != b.exp
       while(/*res.size>0&&*/res.data()[0]==0){--res.size;++res.data();++res.exp;}
@@ -643,36 +690,36 @@ struct Mpzf {
       res.size=0;
       bool carry1=false;
       if(0<yexp){ // must have overlap since x is larger
-	mpn_copyi(rdata, xdata, yexp);
-	xdata+=yexp;
-	absxsize-=yexp;
-	rdata+=yexp;
-	res.size=yexp;
+        mpn_copyi(rdata, xdata, yexp);
+        xdata+=yexp;
+        absxsize-=yexp;
+        rdata+=yexp;
+        res.size=yexp;
       }
       else if(0<xexp){
-	if(absysize<=xexp){ // no overlap
-	  mpn_neg(rdata, ydata, absysize); // assert that it returns 1
-	  rdata+=absysize;
-	  rdata=Mpzf_impl::fill_n_ptr(rdata,xexp-absysize,-1);
-	  mpn_sub_1(rdata, xdata, absxsize, 1);
-	  res.size=absxsize+xexp;
-	  if(res.data()[res.size-1]==0) --res.size;
-	  if(xsize<0) res.size=-res.size;
-	  return res;
-	} else {
-	  mpn_neg(rdata, ydata, xexp); // assert that it returns 1
-	  ydata+=xexp;
-	  absysize-=xexp;
-	  rdata+=xexp;
-	  res.size=xexp;
-	  carry1=true; // assumes no trailing zeros
-	}
+        if(absysize<=xexp){ // no overlap
+          mpn_neg(rdata, ydata, absysize); // assert that it returns 1
+          rdata+=absysize;
+          rdata=Mpzf_impl::fill_n_ptr(rdata,xexp-absysize,-1);
+          mpn_sub_1(rdata, xdata, absxsize, 1);
+          res.size=absxsize+xexp;
+          while(/*res.size>0&&*/res.data()[res.size-1]==0) --res.size;
+          if(xsize<0) res.size=-res.size;
+          return res;
+        } else {
+          mpn_neg(rdata, ydata, xexp); // assert that it returns 1
+          ydata+=xexp;
+          absysize-=xexp;
+          rdata+=xexp;
+          res.size=xexp;
+          carry1=true; // assumes no trailing zeros
+        }
       }
       CGAL_assertion_code( mp_limb_t carry= )
-	mpn_sub(rdata, xdata, absxsize, ydata, absysize);
+        mpn_sub(rdata, xdata, absxsize, ydata, absysize);
       if(carry1)
-	CGAL_assertion_code( carry+= )
-	  mpn_sub_1(rdata, rdata, absxsize, 1);
+        CGAL_assertion_code( carry+= )
+          mpn_sub_1(rdata, rdata, absxsize, 1);
       CGAL_assertion(carry==0);
       res.size+=absxsize;
       while(/*res.size>0&&*/res.data()[res.size-1]==0) --res.size;
@@ -723,7 +770,11 @@ struct Mpzf {
     return res;
   }
 
+#ifndef CGAL_MPZF_DIVISION_OPERATOR
+  friend Mpzf division(Mpzf const&a, Mpzf const&b){
+#else // CGAL_MPZF_DIVISION_OPERATOR
   friend Mpzf operator/(Mpzf const&a, Mpzf const&b){
+#endif // CGAL_MPZF_DIVISION_OPERATOR
     // FIXME: Untested
     int asize=std::abs(a.size);
     int bsize=std::abs(b.size);
@@ -741,16 +792,16 @@ struct Mpzf {
       --res.size;
       mpn_tdiv_qr(qp, rp, 0, adata, asize, bdata, bsize);
       CGAL_assertion_code(
-	  for (int i=0; i<bsize; ++i)
-	    if (rp[i] != 0) throw std::logic_error("non exact Mpzf division");
+          for (int i=0; i<bsize; ++i)
+            if (rp[i] != 0) throw std::logic_error("non exact Mpzf division");
       )
     }
     else if(adata[-1]==0){ // We are lucky
       --adata; ++asize; --res.exp;
       mpn_tdiv_qr(qp, rp, 0, adata, asize, bdata, bsize);
       CGAL_assertion_code(
-	  for (int i=0; i<bsize; ++i)
-	    if (rp[i] != 0) throw std::logic_error("non exact Mpzf division");
+          for (int i=0; i<bsize; ++i)
+            if (rp[i] != 0) throw std::logic_error("non exact Mpzf division");
       )
     }
     else{
@@ -763,8 +814,8 @@ struct Mpzf {
       //a2.exp = a.exp-1;
       mpn_tdiv_qr(qp, rp, 0, a2.data(), asize+1, bdata, bsize);
       CGAL_assertion_code(
-	  for (int i=0; i<bsize; ++i)
-	    if (rp[i] != 0) throw std::logic_error("non exact Mpzf division");
+          for (int i=0; i<bsize; ++i)
+            if (rp[i] != 0) throw std::logic_error("non exact Mpzf division");
       )
     }
     while(/*res.size>0&&*/res.data()[res.size-1]==0) --res.size;
@@ -796,9 +847,9 @@ struct Mpzf {
     else { mpn_copyi(res.data(), b.data(), bsize); }
     res.exp = 0; // Pick b.exp? or the average? 0 helps return 1 more often.
     if (asize < bsize)
-      res.size = mpn_gcd(res.data(), res.data(), bsize, tmp.data(), asize);
+      res.size = Size_type(mpn_gcd(res.data(), res.data(), bsize, tmp.data(), asize));
     else
-      res.size = mpn_gcd(res.data(), tmp.data(), asize, res.data(), bsize);
+      res.size = Size_type(mpn_gcd(res.data(), tmp.data(), asize, res.data(), bsize));
     if(rtz!=0) {
       mp_limb_t c = mpn_lshift(res.data(), res.data(), res.size, rtz);
       if(c) { res.data()[res.size]=c; ++res.size; }
@@ -847,6 +898,9 @@ struct Mpzf {
     }
   }
 
+  friend Mpzf operator+(Mpzf const&x){
+    return x;
+  }
   friend Mpzf operator-(Mpzf const&x){
     Mpzf ret = x;
     ret.size = -ret.size;
@@ -855,6 +909,9 @@ struct Mpzf {
   Mpzf& operator+=(Mpzf const&x){ *this=*this+x; return *this; }
   Mpzf& operator-=(Mpzf const&x){ *this=*this-x; return *this; }
   Mpzf& operator*=(Mpzf const&x){ *this=*this*x; return *this; }
+#ifdef CGAL_MPZF_DIVISION_OPERATOR
+  Mpzf& operator/=(Mpzf const&x){ *this=*this/x; return *this; }
+#endif // not CGAL_MPZF_DIVISION_OPERATOR
 
   bool is_canonical () const {
     if (size == 0) return true;
@@ -879,7 +936,7 @@ struct Mpzf {
     if(size==0) return 0;
     int asize = std::abs(size);
     mp_limb_t top = data()[asize-1];
-    double dtop = top;
+    double dtop = (double)top;
     if(top >= (1LL<<53) || asize == 1) /* ok */ ;
     else { dtop += (double)data()[asize-2] * ldexp(1.,-GMP_NUMB_BITS); }
     return ldexp( (size<0) ? -dtop : dtop, (asize-1+exp) * GMP_NUMB_BITS);
@@ -895,16 +952,16 @@ struct Mpzf {
     int lz = Mpzf_impl::clz(x);
     if (lz <= 11) {
       if (lz != 11) {
-	e += (11 - lz);
-	x >>= (11 - lz);
+        e += (11 - lz);
+        x >>= (11 - lz);
       }
-      dl = x;
-      dh = x + 1;
+      dl = double(x);
+      dh = double(x + 1);
       // Check for the few cases where dh=x works (asize==1 and the evicted
       // bits from x were 0s)
     }
     else if (asize == 1) {
-      dl = dh = x; // conversion is exact
+      dl = dh = double(x); // conversion is exact
     }
     else {
       mp_limb_t y = data()[asize-2];
@@ -912,8 +969,8 @@ struct Mpzf {
       x <<= (lz - 11);
       y >>= (75 - lz);
       x |= y;
-      dl = x;
-      dh = x + 1;
+      dl = double(x);
+      dh = double(x + 1);
       // Check for the few cases where dh=x works (asize==2 and the evicted
       // bits from y were 0s)
     }
@@ -927,9 +984,7 @@ struct Mpzf {
   }
 
 #ifdef CGAL_USE_GMPXX
-#ifndef CGAL_CFG_NO_CPP0X_EXPLICIT_CONVERSION_OPERATORS
   explicit
-#endif
   operator mpq_class () const {
     mpq_class q;
     export_to_mpq_t(q.get_mpq_t());
@@ -937,9 +992,7 @@ struct Mpzf {
   }
 #endif
 
-#ifndef CGAL_CFG_NO_CPP0X_EXPLICIT_CONVERSION_OPERATORS
   explicit
-#endif
   operator Gmpq () const {
     Gmpq q;
     export_to_mpq_t(q.mpq());
@@ -950,25 +1003,23 @@ struct Mpzf {
     CGAL_precondition(mpq_cmp_ui(q,0,1)==0);
     if (size != 0) {
       mpz_import (mpq_numref (q),
-		  std::abs(size),
-		  -1, // data()[0] is the least significant part
-		  sizeof(mp_limb_t),
-		  0, // native endianness inside mp_limb_t
-		  GMP_NAIL_BITS, // should be 0
-		  data());
+                  std::abs(size),
+                  -1, // data()[0] is the least significant part
+                  sizeof(mp_limb_t),
+                  0, // native endianness inside mp_limb_t
+                  GMP_NAIL_BITS, // should be 0
+                  data());
       if (exp > 0)
-	mpq_mul_2exp(q, q, (sizeof(mp_limb_t) * CHAR_BIT *  exp));
+        mpq_mul_2exp(q, q, (sizeof(mp_limb_t) * CHAR_BIT *  exp));
       else if (exp < 0)
-	mpq_div_2exp(q, q, (sizeof(mp_limb_t) * CHAR_BIT * -exp));
+        mpq_div_2exp(q, q, (sizeof(mp_limb_t) * CHAR_BIT * -exp));
 
       if (size < 0)
-	mpq_neg(q,q);
+        mpq_neg(q,q);
     }
   }
 #if 0
-#ifndef CGAL_CFG_NO_CPP0X_EXPLICIT_CONVERSION_OPERATORS
   explicit
-#endif
 // This makes Mpzf==int ambiguous
   operator Gmpzf () const {
     mpz_t z;
@@ -1013,96 +1064,100 @@ std::istream& operator>> (std::istream& is, Mpzf& a)
       typedef Tag_false            Is_numerical_sensitive;
 
       struct Is_zero
-	: public std::unary_function< Type, bool > {
-	  bool operator()( const Type& x ) const {
-	    return x.is_zero();
-	  }
-	};
+        : public CGAL::cpp98::unary_function< Type, bool > {
+          bool operator()( const Type& x ) const {
+            return x.is_zero();
+          }
+        };
 
       struct Is_one
-	: public std::unary_function< Type, bool > {
-	  bool operator()( const Type& x ) const {
-	    return x.is_one();
-	  }
-	};
+        : public CGAL::cpp98::unary_function< Type, bool > {
+          bool operator()( const Type& x ) const {
+            return x.is_one();
+          }
+        };
 
       struct Gcd
-	: public std::binary_function< Type, Type, Type > {
-	  Type operator()(
-	      const Type& x,
-	      const Type& y ) const {
-	    return Mpzf_gcd(x, y);
-	  }
-	};
+        : public CGAL::cpp98::binary_function< Type, Type, Type > {
+          Type operator()(
+              const Type& x,
+              const Type& y ) const {
+            return Mpzf_gcd(x, y);
+          }
+        };
 
       struct Square
-	: public std::unary_function< Type, Type > {
-	  Type operator()( const Type& x ) const {
-	    return Mpzf_square(x);
-	  }
-	};
+        : public CGAL::cpp98::unary_function< Type, Type > {
+          Type operator()( const Type& x ) const {
+            return Mpzf_square(x);
+          }
+        };
 
       struct Integral_division
-	: public std::binary_function< Type, Type, Type > {
-	  Type operator()(
-	      const Type& x,
-	      const Type& y ) const {
-	    return x / y;
-	  }
-	};
+        : public CGAL::cpp98::binary_function< Type, Type, Type > {
+          Type operator()(
+              const Type& x,
+              const Type& y ) const {
+#ifdef CGAL_MPZF_DIVISION_OPERATOR
+            return x / y;
+#else // not CGAL_MPZF_DIVISION_OPERATOR
+            return division(x, y);
+#endif // not CGAL_MPZF_DIVISION_OPERATOR
+          }
+        };
 
       struct Sqrt
-	: public std::unary_function< Type, Type > {
-	  Type operator()( const Type& x) const {
-	    return Mpzf_sqrt(x);
-	  }
-	};
+        : public CGAL::cpp98::unary_function< Type, Type > {
+          Type operator()( const Type& x) const {
+            return Mpzf_sqrt(x);
+          }
+        };
 
       struct Is_square
-	: public std::binary_function< Type, Type&, bool > {
-	  bool operator()( const Type& x, Type& y ) const {
-	    // TODO: avoid doing 2 calls.
-	    if (!Mpzf_is_square(x)) return false;
-	    y = Mpzf_sqrt(x);
-	    return true;
-	  }
-	  bool operator()( const Type& x) const {
-	    return Mpzf_is_square(x);
-	  }
-	};
+        : public CGAL::cpp98::binary_function< Type, Type&, bool > {
+          bool operator()( const Type& x, Type& y ) const {
+            // TODO: avoid doing 2 calls.
+            if (!Mpzf_is_square(x)) return false;
+            y = Mpzf_sqrt(x);
+            return true;
+          }
+          bool operator()( const Type& x) const {
+            return Mpzf_is_square(x);
+          }
+        };
 
     };
   template <> struct Real_embeddable_traits< Mpzf >
     : public INTERN_RET::Real_embeddable_traits_base< Mpzf , CGAL::Tag_true > {
       struct Sgn
-	: public std::unary_function< Type, ::CGAL::Sign > {
-	  ::CGAL::Sign operator()( const Type& x ) const {
-	    return x.sign();
-	  }
-	};
+        : public CGAL::cpp98::unary_function< Type, ::CGAL::Sign > {
+          ::CGAL::Sign operator()( const Type& x ) const {
+            return x.sign();
+          }
+        };
 
       struct To_double
-	: public std::unary_function< Type, double > {
-	    double operator()( const Type& x ) const {
-	      return x.to_double();
-	    }
-	};
+        : public CGAL::cpp98::unary_function< Type, double > {
+            double operator()( const Type& x ) const {
+              return x.to_double();
+            }
+        };
 
       struct Compare
-	: public std::binary_function< Type, Type, Comparison_result > {
-	    Comparison_result operator()(
-		const Type& x,
-		const Type& y ) const {
-	      return CGAL::sign(Mpzf_cmp(x,y));
-	    }
-	};
+        : public CGAL::cpp98::binary_function< Type, Type, Comparison_result > {
+            Comparison_result operator()(
+                const Type& x,
+                const Type& y ) const {
+              return CGAL::sign(Mpzf_cmp(x,y));
+            }
+        };
 
       struct To_interval
-	: public std::unary_function< Type, std::pair< double, double > > {
-	    std::pair<double, double> operator()( const Type& x ) const {
-	      return x.to_interval();
-	    }
-	};
+        : public CGAL::cpp98::unary_function< Type, std::pair< double, double > > {
+            std::pair<double, double> operator()( const Type& x ) const {
+              return x.to_interval();
+            }
+        };
 
     };
 
@@ -1119,6 +1174,36 @@ CGAL_DEFINE_COERCION_TRAITS_FROM_TO(mpz_class,Mpzf)
 
 }
 
+/* There isn't much Eigen can do with such a type,
+ * mostly this is here for IsInteger to protect people.
+ */
+namespace Eigen {
+  template<class> struct NumTraits;
+  template<> struct NumTraits<CGAL::Mpzf>
+  {
+    typedef CGAL::Mpzf Real;
+    /* Should this be Quotient<Mpzf>? Gmpq?  */
+    typedef CGAL::Mpzf NonInteger;
+    typedef CGAL::Mpzf Nested;
+    typedef CGAL::Mpzf Literal;
+
+    static inline Real epsilon() { return 0; }
+    static inline Real dummy_precision() { return 0; }
+
+    enum {
+      /* Only exact divisions are supported, close enough to an integer.
+       * This way we get compilation failures instead of runtime.  */
+      IsInteger = 1,
+      IsSigned = 1,
+      IsComplex = 0,
+      RequireInitialization = 1,
+      ReadCost = 6,
+      AddCost = 30,
+      MulCost = 50
+    };
+  };
+}
+
 #if defined(BOOST_MSVC)
 #  pragma warning(pop)
 #endif
