@@ -1,26 +1,23 @@
 // Copyright (c) 1997-2000  Max-Planck-Institute Saarbruecken (Germany).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
 //
-// Licensees holding a valid commercial license may use this file in
-// accordance with the commercial license agreement provided with the software.
-//
-// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.5-branch/Nef_2/include/CGAL/Nef_2/PM_point_locator.h $
-// $Id: PM_point_locator.h 45448 2008-09-09 16:03:25Z spion $
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Nef_2/include/CGAL/Nef_2/PM_point_locator.h $
+// $Id: PM_point_locator.h a358aac0b1c 2023-08-28T13:40:10+02:00 Sebastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Michael Seel <seel@mpi-sb.mpg.de>
 #ifndef CGAL_PM_POINT_LOCATOR_H
 #define CGAL_PM_POINT_LOCATOR_H
 
+#include <CGAL/license/Nef_2.h>
+
+
 #include <CGAL/basic.h>
 #include <CGAL/Unique_hash_map.h>
+#include <CGAL/use.h>
 #include <CGAL/Nef_2/Constrained_triang_traits.h>
 #include <CGAL/Nef_2/Object_handle.h>
 #include <CGAL/Circulator_project.h>
@@ -28,9 +25,13 @@
 #undef CGAL_NEF_DEBUG
 #define CGAL_NEF_DEBUG 17
 #include <CGAL/Nef_2/debug.h>
+#ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
 #include <CGAL/Nef_2/geninfo.h>
+#else
+#include <boost/any.hpp>
+#endif
 
-#ifdef CGAL_USE_LEDA
+#ifdef CGAL_USE_LEDA_LIBRARY
 #include <CGAL/LEDA_basic.h>
 # if __LEDA__ > 410 && __LEDA__ < 441
 #  define CGAL_USING_PPL
@@ -38,7 +39,7 @@
 # endif
 #endif
 
-CGAL_BEGIN_NAMESPACE
+namespace CGAL {
 
 enum object_kind { VERTEX, EDGE_CROSSING, EDGE_COLLINEAR };
 
@@ -139,6 +140,20 @@ public:
   using Base::number_of_vertices;
   using Base::number_of_halfedges;
   using Base::number_of_faces;
+  using Base::info;
+  using Base::is_closed_at_source;
+  using Base::source;
+  using Base::target;
+  using Base::cyclic_adj_succ;
+  using Base::mark;
+  using Base::twin;
+  using Base::flip_diagonal;
+  using Base::is_isolated;
+  using Base::first_out_edge;
+  using Base::next;
+  using Base::previous;
+  using Base::face;
+  using Base::point;
 
   Halfedge_const_handle out_wedge(Vertex_const_handle v,
     const Direction& d, bool& collinear) const
@@ -150,13 +165,13 @@ public:
   { CGAL_NEF_TRACEN("out_wedge "<<PV(v));
     CGAL_assertion(!is_isolated(v));
     collinear=false;
-    Point p = point(v);
     Halfedge_const_handle e_res = first_out_edge(v);
     Direction d_res = direction(e_res);
     Halfedge_around_vertex_const_circulator el(e_res),ee(el);
     CGAL_For_all(el,ee) {
-      if ( K.strictly_ordered_ccw(d_res, direction(el), d) )
+      if ( K.strictly_ordered_ccw(d_res, direction(el), d) ){
         e_res = el; d_res = direction(e_res);
+      }
     }
     CGAL_NEF_TRACEN("  determined "<<PE(e_res)<<" "<<d_res);
     if ( direction(cyclic_adj_succ(e_res)) == d ) {
@@ -175,8 +190,6 @@ public:
 
   /*{\Mcreation 3}*/
 
-  PM_naive_point_locator() : Base() {}
-
   /*{\Moptions constref=yes}*/
   PM_naive_point_locator(const Plane_map& P, const Geometry& k = Geometry()) :
     Base(const_cast<Plane_map&>(P)), K(k) {}
@@ -205,7 +218,7 @@ public:
   { CGAL_NEF_TRACEN("locate naivly "<<s);
     if (this->number_of_vertices() == 0)
       CGAL_error_msg("PM_naive_point_locator: plane map is empty.");
-    Point p = K.source(s);
+    const Point& p = K.source(s);
     Vertex_const_iterator vit;
     for(vit = this->vertices_begin(); vit != this->vertices_end(); ++vit) {
       if ( p == point(vit) ) return make_object(vit);
@@ -222,7 +235,7 @@ public:
     Direction dso = K.construct_direction(K.target(s),p), d_res;
     CGAL::Unique_hash_map<Halfedge_const_handle,bool> visited(false);
     for(vit = this->vertices_begin(); vit != this->vertices_end(); ++vit) {
-      Point p_res, vp = point(vit);
+      const Point& vp = point(vit);
       if ( K.contains(ss,vp) ) {
         CGAL_NEF_TRACEN(" location via vertex at "<<vp);
         ss = K.construct_segment(p,vp); // we shrink the segment
@@ -246,8 +259,8 @@ public:
 
     for (eit = this->halfedges_begin(); eit != this->halfedges_end(); ++eit) {
       if ( visited[eit] ) continue;
-      Point se = point(source(eit)),
-            te = point(target(eit));
+      const Point& se = point(source(eit));
+      const Point& te = point(target(eit));
       int o1 = K.orientation(ss,se);
       int o2 = K.orientation(ss,te);
       if ( o1 == -o2 && // internal intersection
@@ -280,24 +293,24 @@ public:
   |bool operator() (const Vertex_/Halfedge_/Face_const_handle&)|.\\
   The object returned is intersected by the segment |s| and has
   minimal distance to |s.source()| and |M(h)| holds on the converted
-  object. The operation returns the null handle |NULL| if the ray shoot
+  object. The operation returns the null handle |nullptr| if the ray shoot
   along |s| does not hit any object |h| of |P| with |M(h)|.}*/
   { CGAL_NEF_TRACEN("naive ray_shoot "<<s);
     CGAL_assertion( !K.is_degenerate(s) );
-    Point p = K.source(s);
+    const Point& p = K.source(s);
     Segment ss(s);
     Direction d = K.construct_direction(K.source(s),K.target(s));
     Object_handle h = locate(s);
     Vertex_const_handle v;
     Halfedge_const_handle e;
     Face_const_handle f;
-    if ( assign(v,h) && M(v) ||
-         assign(e,h) && M(e) ||
-         assign(f,h) && M(f) ) return h;
+    if ( ( assign(v,h) && M(v) ) ||
+         ( assign(e,h) && M(e) ) ||
+         ( assign(f,h) && M(f) ) ) return h;
     h = Object_handle();
     CGAL_NEF_TRACEN("not contained");
     for (v = this->vertices_begin(); v != this->vertices_end(); ++v) {
-      Point pv = point(v);
+      const Point& pv = point(v);
       if ( !K.contains(ss,pv) ) continue;
       CGAL_NEF_TRACEN("candidate "<<pv);
       if ( M(v) ) {
@@ -429,6 +442,16 @@ public:
   using Base::K;
   using Base::number_of_vertices;
   using Base::faces_begin;
+  using Base::info;
+  using Base::flip_diagonal;
+  using Base::twin;
+  using Base::next;
+  using Base::previous;
+  using Base::source;
+  using Base::target;
+  using Base::point;
+  using Base::segment;
+  using Base::face;
 
   /*{\Mtypes 2}*/
   /*{\Mtext All local types of |PM_naive_point_locator| are inherited.}*/
@@ -437,31 +460,65 @@ public:
 
   struct CT_link_to_original : Decorator { // CT decorator
     const Decorator& Po;
+    using Decorator::info;
+
     CT_link_to_original(const Decorator& P, const Decorator& Poi)
       : Decorator(P), Po(Poi) {}
+
     void operator()(Vertex_handle vn, Vertex_const_handle vo) const
     { Face_const_handle f;
       if ( Po.is_isolated(vo) ) f = Po.face(vo);
+      #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
       geninfo<VF_pair>::create(info(vn));
       geninfo<VF_pair>::access(info(vn)) = VF_pair(vo,f);
+      #else
+      info(vn) = VF_pair(vo,f);
+      #endif
       CGAL_NEF_TRACEN("linking to org "<<PV(vn));
     }
+
     void operator()(Halfedge_handle hn, Halfedge_const_handle ho) const
-    { geninfo<EF_pair>::create(info(hn));
+    {
+      #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
+      geninfo<EF_pair>::create(info(hn));
       geninfo<EF_pair>::access(info(hn)) = EF_pair(ho,Po.face(ho));
+      #else
+      info(hn) = EF_pair(ho,Po.face(ho));
+      #endif
       CGAL_NEF_TRACEN("linking to org "<<PE(hn));
     }
   };
 
 protected:
   Vertex_const_handle input_vertex(Vertex_const_handle v) const
-  { return geninfo<VF_pair>::const_access(CT.info(v)).first; }
+  {
+    #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
+    return geninfo<VF_pair>::const_access(CT.info(v)).first;
+    #else
+    return
+      boost::any_cast<VF_pair>(CT.info(v)).first;
+    #endif
+  }
 
   Halfedge_const_handle input_halfedge(Halfedge_const_handle e) const
-  { return geninfo<EF_pair>::const_access(CT.info(e)).first; }
+  {
+    #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
+    return geninfo<EF_pair>::const_access(CT.info(e)).first;
+    #else
+    return
+      boost::any_cast<EF_pair>(CT.info(e)).first;
+    #endif
+  }
 
   Face_const_handle input_face(Halfedge_const_handle e) const
-  { return geninfo<EF_pair>::const_access(CT.info(e)).second; }
+  {
+    #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
+    return geninfo<EF_pair>::const_access(CT.info(e)).second;
+    #else
+    return
+      boost::any_cast<EF_pair>(CT.info(e)).second;
+    #endif
+  }
 
 
   Object_handle input_object(Vertex_const_handle v) const
@@ -496,27 +553,43 @@ protected:
   obstacles and the additional triangulation edges as non-obstacle
   edges. Let's call the sum of the lengths of all edges of the
   triangulation its weight. If the calculated triangulation
-  approximates\footnote{The calculation of general
+  approximates\cgalFootnote{The calculation of general
   minimum-weight-triangulations is conjectured to be NP-complete and
   locally-minimum-weight-triangulations that we use are considered good
   approximations.} the minimum weight triangulation of the obstacle set
-  then the stepping quotient\footnote {The number of non-obstacle edges
+  then the stepping quotient\cgalFootnote {The number of non-obstacle edges
   crossed until an obstacle edge is hit.} for a random direction of the
   ray shot is expected to be $O( \sqrt{n} )$.}*/
 
 
   struct CT_new_edge : Decorator {
     const Decorator& _DP;
+    using Decorator::mark;
+    using Decorator::previous;
+    using Decorator::is_closed_at_source;
+    using Decorator::info;
+    using Decorator::source;
+    using Decorator::twin;
+
     CT_new_edge(const Decorator& CT, const Decorator& DP) :
       Decorator(CT), _DP(DP) {}
     void operator()(Halfedge_handle& e) const
     { Halfedge_handle e_from = previous(e);
       Face_const_handle f;
       if ( is_closed_at_source(e) ) // source(e) was isolated before
+        #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
         f = geninfo<VF_pair>::access(info(source(e))).second;
       else
         f = geninfo<EF_pair>::access(info(e_from)).second;
+        #else
+        f =
+          boost::any_cast<VF_pair>(info(source(e))).second;
+      else
+        f =
+          boost::any_cast<EF_pair>(info(e_from)).second;
+        #endif
       mark(e) = _DP.mark(f);
+      #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
       geninfo<EF_pair>::create(info(e));
       geninfo<EF_pair>::create(info(twin(e)));
 
@@ -526,6 +599,10 @@ protected:
 
       geninfo<EF_pair>::access(info(e)).second =
       geninfo<EF_pair>::access(info(twin(e))).second = f;
+      #else
+      info(e)=EF_pair(Halfedge_const_handle(),f);
+      info(twin(e))=EF_pair(Halfedge_const_handle(),f);
+      #endif
       CGAL_NEF_TRACEN("CT_new_edge "<<PE(e));
     }
   };
@@ -565,10 +642,10 @@ protected:
       Halfedge_handle e3 = next(e);
       // e1,e3: edges of quadrilateral with diagonal e
 
-      Point a = point(source(e1));
-      Point b = point(target(e1));
-      Point c = point(source(e3));
-      Point d = point(target(e3));
+      const Point& a = point(source(e1));
+      const Point& b = point(target(e1));
+      const Point& c = point(source(e3));
+      const Point& d = point(target(e3));
 
       if (! (this->K.orientation(b,d,a) > 0 && // left_turn
              this->K.orientation(b,d,c) < 0) ) // right_turn
@@ -588,6 +665,7 @@ protected:
 
 
     }
+    CGAL_USE(flip_count);
     CGAL_NEF_TRACEN("  flipped "<<flip_count);
   }
 
@@ -641,18 +719,18 @@ public:
   }
 
   bool ray_shoot_from_outer_facet(Segment& , object_kind& ,
-				  Vertex_const_handle &,
-				  Halfedge_const_handle& ,
-				  const Tag_true& ) const {
+                                  Vertex_const_handle &,
+                                  Halfedge_const_handle& ,
+                                  const Tag_true& ) const {
     return false;
   }
 
   bool ray_shoot_from_outer_facet(Segment& s, object_kind& current,
-				  Vertex_const_handle &v,
-				  Halfedge_const_handle& e,
-				  const Tag_false& ) const {
+                                  Vertex_const_handle &v,
+                                  Halfedge_const_handle& e,
+                                  const Tag_false& ) const {
     CGAL_NEF_TRACEN("target on outer facet");
-    Point p = this->K.source(s);
+    const Point& p = this->K.source(s);
     Vertex_const_handle v1 = CT.vertices_begin();
     Halfedge_const_handle e1 = CT.twin(CT.first_out_edge(v1));
     Halfedge_around_face_const_circulator circ(e1), end(circ);
@@ -660,30 +738,30 @@ public:
     Segment seg;
     bool found = false;
     CGAL_For_all(circ, end) {
-      //	std::cerr << s << std::endl;
-      //	std::cerr << point(source(circ)) << "->" << point(target(circ)) << std::endl;
+      //        std::cerr << s << std::endl;
+      //        std::cerr << point(source(circ)) << "->" << point(target(circ)) << std::endl;
       Object o = intersection(s, Segment(point(source(circ)),
-					 point(target(circ))));
+                                         point(target(circ))));
 
       if(assign(i,o)) {
-	CGAL_NEF_TRACEN("intersection in point " << i);
-	found = true;
-	s = Segment(p,i);
-	if(i == point(source(circ))) {
-	  current = VERTEX;
-	  v = source(circ);
-	} else if(i == point(target(circ))) {
-	  current = VERTEX;
-	  v = target(circ);
-	} else {
-	  current = EDGE_CROSSING;
-	  e = circ;
-	}
+        CGAL_NEF_TRACEN("intersection in point " << i);
+        found = true;
+        s = Segment(p,i);
+        if(i == point(source(circ))) {
+          current = VERTEX;
+          v = source(circ);
+        } else if(i == point(target(circ))) {
+          current = VERTEX;
+          v = target(circ);
+        } else {
+          current = EDGE_CROSSING;
+          e = circ;
+        }
       } else if(assign(seg,o)) {
-	found = true;
-	CGAL_NEF_TRACEN("overlap of segments");
-	current = EDGE_COLLINEAR;
-	e = circ;
+        found = true;
+        CGAL_NEF_TRACEN("overlap of segments");
+        current = EDGE_COLLINEAR;
+        e = circ;
       }
     }
     return found;
@@ -698,12 +776,12 @@ public:
   |bool operator() (const Vertex_/ Halfedge_/Face_const_handle&) const|.\\
   The object returned is intersected by the segment |s| and has minimal
   distance to |s.source()| and |M(h)| holds on the converted object. The
-  operation returns the null handle |NULL| if the ray shoot along |s|
+  operation returns the null handle |nullptr| if the ray shoot along |s|
   does not hit any object |h| of |P| with |M(h)|.}*/
   { Segment s(ss);
     CGAL_NEF_TRACEN("ray_shoot "<<s);
     CGAL_assertion( !this->K.is_degenerate(s) );
-    Point p = this->K.source(s);
+    const Point& p = this->K.source(s);
     Direction d = this->K.construct_direction(p,s.target());
     Vertex_const_handle v;
     Halfedge_const_handle e;
@@ -734,9 +812,9 @@ public:
         if ( M(input_face(e)) ) // face mark
           return make_object(input_face(e));
 
-        Point p1 = CT.point(CT.source(e)),
-              p2 = CT.point(CT.target(e)),
-              p3 = CT.point(CT.target(next(e)));
+        const Point& p1 = CT.point(CT.source(e));
+        const Point& p2 = CT.point(CT.target(e));
+        const Point& p3 = CT.point(CT.target(next(e)));
         int or1 = this->K.orientation(p,s.target(),p1);
         int or2 = this->K.orientation(p,s.target(),p2);
         int or3 = this->K.orientation(p,s.target(),p3);
@@ -758,16 +836,16 @@ public:
     } else {
 
       if(check_tag(typename Is_extended_kernel<Geometry>::value_type())) {
-	CGAL_error_msg( "code is only for Bounded_kernel");
+        CGAL_error_msg( "code is only for Bounded_kernel");
       }
       if(!ray_shoot_from_outer_facet(s,current,v,e,typename Is_extended_kernel<Geometry>::value_type()))
-	return Object_handle();
+        return Object_handle();
     }
 
     CGAL_NEF_TRACEN("current = " << current);
-    if(current == VERTEX)
+    if(current == VERTEX){
       CGAL_NEF_TRACEN(point(v));
-
+    }
     while (true) switch ( current ) {
       case VERTEX:
         { CGAL_NEF_TRACEN("vertex "<<CT.point(v));
@@ -832,12 +910,12 @@ public:
   }
 
   bool within_outer_cycle(Vertex_const_handle ,
-			  const Point& , const Tag_true& ) const {
+                          const Point& , const Tag_true& ) const {
     return true;
   }
 
   bool within_outer_cycle(Vertex_const_handle v,
-			  const Point& q, const Tag_false& ) const {
+                          const Point& q, const Tag_false& ) const {
     typedef Project_halfedge_point<typename Decorator::Halfedge, Point> Project;
     typedef Circulator_project<Halfedge_around_face_const_circulator,
       Project, const Point&, const Point*> Circulator;
@@ -886,11 +964,19 @@ PM_point_locator<PMD,GEO>::
 { CGAL_NEF_TRACEN("clear_static_point_locator");
   Vertex_iterator vit, vend = CT.vertices_end();
   for (vit = CT.vertices_begin(); vit != vend; ++vit) {
+    #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
     geninfo<VF_pair>::clear(CT.info(vit));
+    #else
+    CT.info(vit)=boost::any();
+    #endif
   }
   Halfedge_iterator eit, eend = CT.halfedges_end();
   for (eit = CT.halfedges_begin(); eit != eend; ++eit) {
+    #ifdef CGAL_I_DO_WANT_TO_USE_GENINFO
     geninfo<EF_pair>::clear(CT.info(eit));
+    #else
+    CT.info(eit)=boost::any();
+    #endif
   }
   CT.clear();
   delete &(CT.plane_map());
@@ -912,7 +998,7 @@ PM_point_locator<PMD,GEO>::walk_in_triangulation(const Point& q) const
       return Object_handle();
 
   Halfedge_const_handle e;
-  Point p = CT.point(v);
+  const Point& p = CT.point(v);
   if ( p == q ) return make_object(v);
   //  Segment s = this->K.construct_segment(p,q);
   Direction dir = this->K.construct_direction(p,q);
@@ -964,6 +1050,6 @@ PM_point_locator<PMD,GEO>::walk_in_triangulation(const Point& q) const
   return Object_handle(); // never reached warning acceptable
 }
 
-CGAL_END_NAMESPACE
+} //namespace CGAL
 
 #endif // CGAL_PM_POINT_LOCATOR_H
