@@ -1,27 +1,19 @@
-// Copyright (c) 2000  
+// Copyright (c) 2000, 2016
 // Utrecht University (The Netherlands),
 // ETH Zurich (Switzerland),
 // INRIA Sophia-Antipolis (France),
 // Max-Planck-Institute Saarbruecken (Germany),
-// and Tel-Aviv University (Israel).  All rights reserved. 
+// and Tel-Aviv University (Israel).  All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 3 of the License,
-// or (at your option) any later version.
+// This file is part of CGAL (www.cgal.org)
 //
-// Licensees holding a valid commercial license may use this file in
-// accordance with the commercial license agreement provided with the software.
+// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/Cartesian_kernel/include/CGAL/predicates/kernel_ftC2.h $
+// $Id: kernel_ftC2.h 65ea1e36215 2021-09-29T16:55:36+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
-// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
-// 
+// Author(s)     : Herve Bronnimann, Sylvain Pion, Olivier Devillers
 //
-// Author(s)     : Herve Bronnimann (Herve.Bronnimann@sophia.inria.fr)
-//                 Sylvain Pion
 
 #ifndef CGAL_PREDICATES_KERNEL_FTC2_H
 #define CGAL_PREDICATES_KERNEL_FTC2_H
@@ -63,10 +55,10 @@ equal_lineC2(const FT &l1a, const FT &l1b, const FT &l1c,
         return false; // Not parallel.
     typename Sgn<FT>::result_type s1a = CGAL_NTS sign(l1a);
     if (s1a != ZERO)
-        return s1a == CGAL_NTS sign(l2a)
-	    && sign_of_determinant(l1a, l1c, l2a, l2c) == ZERO;
-    return CGAL_NTS sign(l1b) == CGAL_NTS sign(l2b)
-	&& sign_of_determinant(l1b, l1c, l2b, l2c) == ZERO;
+        return CGAL_AND(s1a == CGAL_NTS sign(l2a),
+                        sign_of_determinant(l1a, l1c, l2a, l2c) == ZERO);
+    return CGAL_AND(CGAL_NTS sign(l1b) == CGAL_NTS sign(l2b),
+                    sign_of_determinant(l1b, l1c, l2b, l2c) == ZERO);
 }
 
 template < class FT >
@@ -208,15 +200,15 @@ compare_y_at_xC2(const FT &px, const FT &py,
     CGAL_kernel_precondition(are_ordered(ssx, px, stx));
 
     if (ssx < stx)
-	return orientationC2(px, py, ssx, ssy, stx, sty);
+        return orientationC2(px, py, ssx, ssy, stx, sty);
     else if (ssx > stx)
-	return orientationC2(px, py, stx, sty, ssx, ssy);
+        return orientationC2(px, py, stx, sty, ssx, ssy);
     else {
-	if (py < (CGAL::min)(sty, ssy))
-	    return SMALLER;
-	if (py > (CGAL::max)(sty, ssy))
-	    return LARGER;
-	return EQUAL;
+        if (py < (CGAL::min)(sty, ssy))
+            return SMALLER;
+        if (py > (CGAL::max)(sty, ssy))
+            return LARGER;
+        return EQUAL;
     }
 }
 
@@ -238,32 +230,32 @@ compare_y_at_x_segment_C2(const FT &px,
     CGAL_kernel_precondition(are_ordered(s1sx, px, s1tx));
     CGAL_kernel_precondition(are_ordered(s2sx, px, s2tx));
 
-    if (s1sx != s1tx && s2sx != s2tx) {
-	FT s1stx = s1sx-s1tx;
-	FT s2stx = s2sx-s2tx;
+    if (CGAL_AND(s1sx != s1tx, s2sx != s2tx)) {
+        FT s1stx = s1sx-s1tx;
+        FT s2stx = s2sx-s2tx;
 
-	return CGAL_NTS compare(s1sx, s1tx) *
-	       CGAL_NTS compare(s2sx, s2tx) *
-	       CGAL_NTS compare(-(s1sx-px)*(s1sy-s1ty)*s2stx,
-		                (s2sy-s1sy)*s2stx*s1stx
-		                -(s2sx-px)*(s2sy-s2ty)*s1stx );
+        return CGAL_NTS compare(s1sx, s1tx) *
+               CGAL_NTS compare(s2sx, s2tx) *
+               CGAL_NTS compare(-(s1sx-px)*(s1sy-s1ty)*s2stx,
+                                (s2sy-s1sy)*s2stx*s1stx
+                                -(s2sx-px)*(s2sy-s2ty)*s1stx );
     }
     else {
-	if (s1sx == s1tx) { // s1 is vertical
-	    typename Compare<FT>::result_type c1, c2;
-	    c1 = compare_y_at_xC2(px, s1sy, s2sx, s2sy, s2tx, s2ty);
-	    c2 = compare_y_at_xC2(px, s1ty, s2sx, s2sy, s2tx, s2ty);
-	    if (c1 == c2)
-		return c1;
-	    return EQUAL;
-	}
-	// s2 is vertical
-	typename Compare<FT>::result_type c3, c4;
-	c3 = compare_y_at_xC2(px, s2sy, s1sx, s1sy, s1tx, s1ty);
-	c4 = compare_y_at_xC2(px, s2ty, s1sx, s1sy, s1tx, s1ty);
-	if (c3 == c4)
-	    return -c3;
-	return EQUAL;
+        if (s1sx == s1tx) { // s1 is vertical
+            typename Compare<FT>::result_type c1, c2;
+            c1 = compare_y_at_xC2(px, s1sy, s2sx, s2sy, s2tx, s2ty);
+            c2 = compare_y_at_xC2(px, s1ty, s2sx, s2sy, s2tx, s2ty);
+            if (c1 == c2)
+                return c1;
+            return EQUAL;
+        }
+        // s2 is vertical
+        typename Compare<FT>::result_type c3, c4;
+        c3 = compare_y_at_xC2(px, s2sy, s1sx, s1sy, s1tx, s1ty);
+        c4 = compare_y_at_xC2(px, s2ty, s1sx, s1sy, s1tx, s1ty);
+        if (c3 == c4)
+            return -c3;
+        return EQUAL;
     }
 }
 
@@ -271,18 +263,18 @@ template < class FT >
 CGAL_KERNEL_MEDIUM_INLINE
 typename Equal_to<FT>::result_type
 equal_directionC2(const FT &dx1, const FT &dy1,
-                  const FT &dx2, const FT &dy2) 
+                  const FT &dx2, const FT &dy2)
 {
-  return CGAL_NTS sign(dx1) == CGAL_NTS sign(dx2)
-      && CGAL_NTS sign(dy1) == CGAL_NTS sign(dy2)
-      && sign_of_determinant(dx1, dy1, dx2, dy2) == ZERO;
+  return CGAL_AND_3( CGAL_NTS sign(dx1) == CGAL_NTS sign(dx2),
+                     CGAL_NTS sign(dy1) == CGAL_NTS sign(dy2),
+                     sign_of_determinant(dx1, dy1, dx2, dy2) == ZERO );
 }
 
 template < class FT >
 CGAL_KERNEL_MEDIUM_INLINE
 typename Compare<FT>::result_type
 compare_angle_with_x_axisC2(const FT &dx1, const FT &dy1,
-                            const FT &dx2, const FT &dy2) 
+                            const FT &dx2, const FT &dy2)
 {
   // angles are in [-pi,pi], and the angle between Ox and d1 is compared
   // with the angle between Ox and d2
@@ -302,16 +294,16 @@ compare_angle_with_x_axisC2(const FT &dx1, const FT &dy1,
 template < class FT >
 CGAL_KERNEL_MEDIUM_INLINE
 typename Compare<FT>::result_type
-compare_slopesC2(const FT &l1a, const FT &l1b, const FT &l2a, const FT &l2b) 
+compare_slopesC2(const FT &l1a, const FT &l1b, const FT &l2a, const FT &l2b)
 {
    typedef typename Compare<FT>::result_type result_type;
 
    if (CGAL_NTS is_zero(l1a))  // l1 is horizontal
     return CGAL_NTS is_zero(l2b) ? result_type(SMALLER)
-	                         : CGAL_NTS sign(l2a) * CGAL_NTS sign(l2b);
+                                 : CGAL_NTS sign(l2a) * CGAL_NTS sign(l2b);
    if (CGAL_NTS is_zero(l2a)) // l2 is horizontal
     return CGAL_NTS is_zero(l1b) ? result_type(LARGER)
-	                         : - CGAL_NTS sign(l1a) * CGAL_NTS sign(l1b);
+                                 : - CGAL_NTS sign(l1a) * CGAL_NTS sign(l1b);
    if (CGAL_NTS is_zero(l1b)) return CGAL_NTS is_zero(l2b) ? EQUAL : LARGER;
    if (CGAL_NTS is_zero(l2b)) return SMALLER;
    result_type l1_sign = - CGAL_NTS sign(l1a) * CGAL_NTS sign(l1b);
@@ -322,18 +314,18 @@ compare_slopesC2(const FT &l1a, const FT &l1b, const FT &l2a, const FT &l2b)
 
    if (l1_sign > ZERO)
      return CGAL_NTS compare ( CGAL_NTS abs(l1a * l2b),
-			       CGAL_NTS abs(l2a * l1b) );
+                               CGAL_NTS abs(l2a * l1b) );
 
    return CGAL_NTS compare ( CGAL_NTS abs(l2a * l1b),
-			     CGAL_NTS abs(l1a * l2b) );
+                             CGAL_NTS abs(l1a * l2b) );
 }
 
 template < class FT >
 CGAL_KERNEL_MEDIUM_INLINE
 typename Compare<FT>::result_type
-compare_slopesC2(const FT &s1_src_x, const FT &s1_src_y, const FT &s1_tgt_x, 
-                 const FT &s1_tgt_y, const FT &s2_src_x, const FT &s2_src_y, 
-                 const FT &s2_tgt_x, const FT &s2_tgt_y) 
+compare_slopesC2(const FT &s1_src_x, const FT &s1_src_y, const FT &s1_tgt_x,
+                 const FT &s1_tgt_y, const FT &s2_src_x, const FT &s2_src_y,
+                 const FT &s2_tgt_x, const FT &s2_tgt_y)
 {
    typedef typename Compare<FT>::result_type  Cmp;
    typedef typename Sgn<FT>::result_type      Sg;
@@ -399,7 +391,9 @@ typename Compare<FT>::result_type
 compare_lexicographically_xyC2(const FT &px, const FT &py,
                                const FT &qx, const FT &qy)
 {
-  typename Compare<FT>::result_type c = CGAL_NTS compare(px,qx);
+  typedef typename Compare<FT>::result_type Cmp;
+  Cmp c = CGAL_NTS compare(px,qx);
+  if (is_indeterminate(c)) return indeterminate<Cmp>();
   return (c != EQUAL) ? c : CGAL_NTS compare(py,qy);
 }
 
@@ -635,6 +629,107 @@ side_of_oriented_lineC2(const FT &a, const FT &b, const FT &c,
   return CGAL_NTS sign(a*x+b*y+c);
 }
 
+template <class FT>
+Comparison_result
+compare_power_distanceC2(const FT& px, const FT& py, const FT& pwt,
+                         const FT& qx, const FT& qy, const FT& qwt,
+                         const FT& rx, const FT& ry)
+{
+  // returns SMALLER if r is closer to p w.r.t. the power metric
+  FT d1 = CGAL_NTS square(rx - px) + CGAL_NTS square(ry - py) - pwt;
+  FT d2 = CGAL_NTS square(rx - qx) + CGAL_NTS square(ry - qy) - qwt;
+  return CGAL_NTS compare(d1, d2);
+}
+
+template <class FT>
+CGAL_KERNEL_MEDIUM_INLINE
+Bounded_side
+power_side_of_bounded_power_circleC2(const FT &px, const FT &py, const FT &pw,
+                                     const FT &qx, const FT &qy, const FT &qw,
+                                     const FT &tx, const FT &ty, const FT &tw)
+{
+  FT dpx = px - qx;
+  FT dpy = py - qy;
+  FT dtx = tx - qx;
+  FT dty = ty - qy;
+  FT dpz = CGAL_NTS square(dpx) + CGAL_NTS square(dpy);
+
+  return enum_cast<Bounded_side>
+    (CGAL_NTS sign(-(CGAL_NTS square(dtx) + CGAL_NTS square(dty)-tw+qw)*dpz
+                   +(dpz-pw+qw)*(dpx*dtx+dpy*dty)));
+}
+
+template <class FT>
+Oriented_side
+power_side_of_oriented_power_circleC2(const FT &px, const FT &py, const FT &pwt,
+                                      const FT &qx, const FT &qy, const FT &qwt,
+                                      const FT &rx, const FT &ry, const FT &rwt,
+                                      const FT &tx, const FT &ty, const FT &twt)
+{
+  // Note: maybe this can be further optimized like the usual in_circle() ?
+
+  // We translate the 4 points so that T becomes the origin.
+  FT dpx = px - tx;
+  FT dpy = py - ty;
+  FT dpz = CGAL_NTS square(dpx) + CGAL_NTS square(dpy) - pwt + twt;
+  FT dqx = qx - tx;
+  FT dqy = qy - ty;
+  FT dqz = CGAL_NTS square(dqx) + CGAL_NTS square(dqy) - qwt + twt;
+  FT drx = rx - tx;
+  FT dry = ry - ty;
+  FT drz = CGAL_NTS square(drx) + CGAL_NTS square(dry) - rwt + twt;
+
+  return sign_of_determinant(dpx, dpy, dpz,
+                             dqx, dqy, dqz,
+                             drx, dry, drz);
+}
+
+template <class FT>
+Oriented_side
+power_side_of_oriented_power_circleC2(const FT &px, const FT &py, const FT &pwt,
+                                      const FT &qx, const FT &qy, const FT &qwt,
+                                      const FT &tx, const FT &ty, const FT &twt)
+{
+  // Same translation as above.
+  FT dpx = px - tx;
+  FT dpy = py - ty;
+  FT dpz = CGAL_NTS square(dpx) + CGAL_NTS square(dpy) - pwt + twt;
+  FT dqx = qx - tx;
+  FT dqy = qy - ty;
+  FT dqz = CGAL_NTS square(dqx) + CGAL_NTS square(dqy) - qwt + twt;
+
+  // We do an orthogonal projection on the (x) axis, if possible.
+  Comparison_result cmpx = CGAL_NTS compare(px, qx);
+  if (cmpx != EQUAL)
+    return cmpx * sign_of_determinant(dpx, dpz, dqx, dqz);
+
+  // If not possible, then on the (y) axis.
+  Comparison_result cmpy = CGAL_NTS compare(py, qy);
+  return cmpy * sign_of_determinant(dpy, dpz, dqy, dqz);
+}
+
+template <class FT>
+Oriented_side
+circumcenter_oriented_side_of_oriented_segmentC2(const FT& ax,  const FT& ay,
+                                                 const FT& bx,  const FT& by,
+                                                 const FT& p0x, const FT& p0y,
+                                                 const FT& p1x, const FT& p1y,
+                                                 const FT& p2x, const FT& p2y)
+{
+  const FT dX = bx - ax;
+  const FT dY = by - ay;
+  const FT R0 = p0x * p0x + p0y * p0y;
+  const FT R1 = p1x * p1x + p1y * p1y;
+  const FT R2 = p2x * p2x + p2y * p2y;
+  const FT denominator = (p1x - p0x) * (p2y - p0y) +
+                         (p0x - p2x) * (p1y - p0y);
+  const FT det = 2 * denominator * (ax * dY - ay * dX)
+                   - (R2 - R1) * (p0x * dX + p0y * dY)
+                   - (R0 - R2) * (p1x * dX + p1y * dY)
+                   - (R1 - R0) * (p2x * dX + p2y * dY);
+  return CGAL::sign(det);
+}
+
 } //namespace CGAL
 
 #endif  // CGAL_PREDICATES_KERNEL_FTC2_H
